@@ -42,6 +42,7 @@
 #include "../storage/ndb/src/ndbapi/NdbQueryBuilder.hpp"
 #include "../storage/ndb/src/ndbapi/NdbQueryOperation.hpp"
 
+#include "ndb_thd.h"
 #include <ndb_version.h>
 
 
@@ -52,7 +53,7 @@
 #define EXPLAIN_NO_PUSH(msgfmt, ...)                              \
 do                                                                \
 {                                                                 \
-  if (unlikely(current_thd->lex->describe))   \
+  if (unlikely(current_thd->lex->describe))                       \
   {                                                               \
     push_warning_printf(current_thd,                              \
                         Sql_condition::SL_NOTE, ER_YES,           \
@@ -300,6 +301,12 @@ ndb_pushed_builder_ctx::ndb_pushed_builder_ctx(const AQP::Join_plan& plan)
       m_tables[i].m_maybe_pushable= 0;
 
       const AQP::Table_access* const table = m_plan.get_table_access(i);
+      if (table->get_table() == NULL)
+      {
+        // There could be unused tables allocated in the 'plan', skip these
+        continue;
+      }
+      
       if (table->get_table()->s->db_type()->db_type != DB_TYPE_NDBCLUSTER)
       {
         DBUG_PRINT("info", ("Table '%s' not in ndb engine, not pushable", 
@@ -369,12 +376,15 @@ ndb_pushed_builder_ctx::ndb_pushed_builder_ctx(const AQP::Join_plan& plan)
     // Fill in table for maping internal <-> external table enumeration
     for (uint i= 0; i < count; i++)
     {
-      const AQP::Table_access* const table = m_plan.get_table_access(i);
-      uint external= table->get_table()->pos_in_table_list->tableno();
-      assert(external < MAX_TABLES);
+      if (m_tables[i].m_maybe_pushable)
+      {
+        const AQP::Table_access* const table = m_plan.get_table_access(i);	
+        const uint external= table->get_table()->pos_in_table_list->tableno();
+        assert(external <  MAX_TABLES);
 
-      m_remap[i].to_external= external;
-      m_remap[external].to_internal= i;
+        m_remap[i].to_external= external;
+        m_remap[external].to_internal= i;
+      }
     }
   }
 } // ndb_pushed_builder_ctx::ndb_pushed_builder_ctx()
@@ -455,7 +465,7 @@ ndb_pushed_builder_ctx::make_pushed_join(
     if (unlikely(error))
       DBUG_RETURN(error);
 
-    const NdbQueryDef* const query_def= m_builder->prepare();
+    const NdbQueryDef* const query_def= m_builder->prepare(get_thd_ndb(current_thd)->ndb);
     if (unlikely(query_def == NULL))
       DBUG_RETURN(-1);  // Get error with ::getNdbError()
 
@@ -590,7 +600,10 @@ ndb_pushed_builder_ctx::is_pushable_as_child(
   
   if ((m_tables[tab_no].m_maybe_pushable & PUSHABLE_AS_CHILD) != PUSHABLE_AS_CHILD)
   {
-    DBUG_PRINT("info", ("Table %s already known 'not is_pushable_as_child'", table->get_table()->alias));
+    if (table->get_table()) //Possible not a real table at all
+    {
+      DBUG_PRINT("info", ("Table %s already known 'not is_pushable_as_child'", table->get_table()->alias));
+    }
     DBUG_RETURN(false);
   }
 
@@ -1276,7 +1289,6 @@ ndb_pushed_builder_ctx::optimize_query_plan()
      * Select set to choose parent from, prefer a 'common'
      * parent if available.
      */
-    uint parent_no;
     ndb_table_access_map const &parents=
        table.m_common_parents.is_clear_all()
        ? table.m_extend_parents
@@ -1286,34 +1298,16 @@ ndb_pushed_builder_ctx::optimize_query_plan()
     assert(!parents.contain(tab_no)); // No circular dependency!
 
     /**
-     * In order to take advantage of the parallelism in the SPJ block;
-     * Initial parent candidate is the first possible among 'parents'.
-     * Will result in the most 'bushy' query plan (aka: star-join)
-     */
-    parent_no= parents.first_table(root_no);
-
-    /**
-     * Push optimization for execution of child operations:
+     * In order to possibly take advantage of the parallelism in the
+     * SPJ block; Choose parent to be the first possible among 'parents'.
+     * Will result in the most 'bushy' query plan (aka: star-join).
      *
-     * To take advantage of the selectivity of parent operations we 
-     * execute any parent operations with fanout <= 1 before this
-     * child operation. By making them depending on parent 
-     * operations with high selectivity, child will be eliminated when
-     * the parent returns no matching rows.
-     *
-     * -> Execute child operation after any such parents
+     * The SPJ block may on its own choose to sequentialize the
+     * execution order of the star-joined children. Thereby eliminating
+     * some operations on its sibling, where INNER_JOIN'ed results could
+     * never be produced.
      */
-    for (uint candidate= parent_no+1; candidate<tab_no; candidate++)
-    {
-      if (parents.contain(candidate))
-      {
-        if (m_tables[candidate].m_fanout > 1.0)
-          break;
-
-        parent_no= candidate;     // Parent candidate is selective, eval after
-      }
-    }
-
+    const uint parent_no= parents.first_table(root_no);
     assert(parent_no < tab_no);
     table.m_parent= parent_no;
     m_tables[parent_no].m_child_fanout*= table.m_fanout*table.m_child_fanout;
