@@ -1,19 +1,25 @@
 /*
-   Copyright (c) 2000, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2000, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /* Testing of the basic functions of a MyISAM table */
 
@@ -68,7 +74,7 @@ static int run_test(const char *filename)
   MI_UNIQUEDEF uniquedef;
   MI_CREATE_INFO create_info;
 
-  bzero((char*) recinfo,sizeof(recinfo));
+  memset(recinfo, 0, sizeof(recinfo));
 
   /* First define 2 columns */
   recinfo[0].type=FIELD_NORMAL; recinfo[0].length=1; /* For NULL bits */
@@ -111,13 +117,13 @@ static int run_test(const char *filename)
   }
   keyinfo[0].flag = (uint8) (pack_keys | unique_key);
 
-  bzero((uchar*) flags,sizeof(flags));
+  memset(flags, 0, sizeof(flags));
   if (opt_unique)
   {
     uint start;
     uniques=1;
-    bzero((char*) &uniquedef,sizeof(uniquedef));
-    bzero((char*) uniqueseg,sizeof(uniqueseg));
+    memset(&uniquedef, 0, sizeof(uniquedef));
+    memset(uniqueseg, 0, sizeof(uniqueseg));
     uniquedef.seg=uniqueseg;
     uniquedef.keysegs=2;
 
@@ -146,7 +152,7 @@ static int run_test(const char *filename)
 
   if (!silent)
     printf("- Creating isam-file\n");
-  bzero((char*) &create_info,sizeof(create_info));
+  memset(&create_info, 0, sizeof(create_info));
   create_info.max_rows=(ulong) (rec_pointer_size ?
 				(1L << (rec_pointer_size*8))/40 :
 				0);
@@ -163,7 +169,7 @@ static int run_test(const char *filename)
   row_count=deleted=0;
   for (i=49 ; i>=1 ; i-=2 )
   {
-    if (insert_count-- == 0) { VOID(mi_close(file)) ; exit(0) ; }
+    if (insert_count-- == 0) { (void) mi_close(file); exit(0) ; }
     j=i%25 +1;
     create_record(record,j);
     error=mi_write(file,record);
@@ -227,7 +233,7 @@ static int run_test(const char *filename)
     found=0;
     while ((error=mi_rrnd(file,read_record,pos)) == 0)
     {
-      if (update_count-- == 0) { VOID(mi_close(file)) ; exit(0) ; }
+      if (update_count-- == 0) { (void) mi_close(file); exit(0) ; }
       memcpy(record,read_record,rec_length);
       update_record(record);
       if (mi_update(file,read_record,record))
@@ -254,7 +260,7 @@ static int run_test(const char *filename)
     for (i=0 ; i <= 10 ; i++)
     {
       /* testing */
-      if (remove_count-- == 0) { VOID(mi_close(file)) ; exit(0) ; }
+      if (remove_count-- == 0) { (void) mi_close(file); exit(0) ; }
       j=i*2;
       if (!flags[j])
 	continue;
@@ -266,14 +272,14 @@ static int run_test(const char *filename)
 	if (verbose || (flags[j] >= 1 ||
 			(error && my_errno != HA_ERR_KEY_NOT_FOUND)))
 	  printf("key: '%.*s'  mi_rkey:  %3d  errno: %3d\n",
-		 (int) key_length,key+test(null_fields),error,my_errno);
+		 (int) key_length,key+MY_TEST(null_fields),error,my_errno);
       }
       else
       {
 	error=mi_delete(file,read_record);
 	if (verbose || error)
 	  printf("key: '%.*s'  mi_delete: %3d  errno: %3d\n",
-		 (int) key_length, key+test(null_fields), error, my_errno);
+		 (int) key_length, key+MY_TEST(null_fields), error, my_errno);
 	if (! error)
 	{
 	  deleted++;
@@ -294,7 +300,7 @@ static int run_test(const char *filename)
 	(error && (flags[i] != 0 || my_errno != HA_ERR_KEY_NOT_FOUND)))
     {
       printf("key: '%.*s'  mi_rkey: %3d  errno: %3d  record: %s\n",
-	     (int) key_length,key+test(null_fields),error,my_errno,record+1);
+	     (int) key_length,key+MY_TEST(null_fields),error,my_errno,record+1);
     }
   }
 
@@ -341,12 +347,12 @@ static void create_key_part(uchar *key,uint rownr)
            keyinfo[0].seg[0].type == HA_KEYTYPE_VARTEXT2)
   {						/* Alpha record */
     /* Create a key that may be easily packed */
-    bfill(key,keyinfo[0].seg[0].length,rownr < 10 ? 'A' : 'B');
+    memset(key, rownr < 10 ? 'A' : 'B', keyinfo[0].seg[0].length);
     sprintf((char*) key+keyinfo[0].seg[0].length-2,"%-2d",rownr);
     if ((rownr & 7) == 0)
     {
       /* Change the key to force a unpack of the next key */
-      bfill(key+3,keyinfo[0].seg[0].length-4,rownr < 10 ? 'a' : 'b');
+      memset(key + 3, rownr < 10 ? 'a' : 'b', keyinfo[0].seg[0].length-4);
     }
   }
   else
@@ -356,7 +362,7 @@ static void create_key_part(uchar *key,uint rownr)
     else
     {
       /* Create a key that may be easily packed */
-      bfill(key,keyinfo[0].seg[0].length,rownr < 10 ? 'A' : 'B');
+      memset(key, rownr < 10 ? 'A' : 'B', keyinfo[0].seg[0].length);
       sprintf((char*) key+keyinfo[0].seg[0].length-2,"%-2d",rownr);
       if ((rownr & 7) == 0)
       {
@@ -399,7 +405,7 @@ static uchar blob_record[MAX_REC_LENGTH+20*20];
 static void create_record(uchar *record,uint rownr)
 {
   uchar *pos;
-  bzero((char*) record,MAX_REC_LENGTH);
+  memset(record, 0, MAX_REC_LENGTH);
   record[0]=1;					/* delete marker */
   if (rownr == 0 && keyinfo[0].seg[0].null_bit)
     record[0]|=keyinfo[0].seg[0].null_bit;	/* Null key */
@@ -413,7 +419,7 @@ static void create_record(uchar *record,uint rownr)
     tmp=strlen((char*) blob_key);
     int4store(pos,tmp);
     ptr=blob_key;
-    memcpy_fixed(pos+4,&ptr,sizeof(char*));
+    memcpy(pos+4, &ptr, sizeof(char*));
     pos+=recinfo[1].length;
   }
   else if (recinfo[1].type == FIELD_VARCHAR)
@@ -441,7 +447,7 @@ static void create_record(uchar *record,uint rownr)
     tmp=strlen((char*) blob_record);
     int4store(pos,tmp);
     ptr=blob_record;
-    memcpy_fixed(pos+4,&ptr,sizeof(char*));
+    memcpy(pos+4, &ptr, sizeof(char*));
   }
   else if (recinfo[2].type == FIELD_VARCHAR)
   {
@@ -470,10 +476,10 @@ static void update_record(uchar *record)
     uchar *column,*ptr;
     int length;
     length=uint4korr(pos);			/* Long blob */
-    memcpy_fixed(&column,pos+4,sizeof(char*));
+    memcpy(&column, pos+4, sizeof(char*));
     memcpy(blob_key,column,length);		/* Move old key */
     ptr=blob_key;
-    memcpy_fixed(pos+4,&ptr,sizeof(char*));	/* Store pointer to new key */
+    memcpy(pos+4, &ptr, sizeof(char*));	/* Store pointer to new key */
     if (keyinfo[0].seg[0].type != HA_KEYTYPE_NUM)
       default_charset_info->cset->casedn(default_charset_info,
                                          (char*) blob_key, length,
@@ -503,20 +509,21 @@ static void update_record(uchar *record)
     uchar *column;
     int length;
     length=uint4korr(pos);
-    memcpy_fixed(&column,pos+4,sizeof(char*));
+    memcpy(&column, pos+4, sizeof(char*));
     memcpy(blob_record,column,length);
-    bfill(blob_record+length,20,'.');	/* Make it larger */
+    memset(blob_record + length, '.', 20);      /* Make it larger */
     length+=20;
     int4store(pos,length);
     column= blob_record;
-    memcpy_fixed(pos+4,&column,sizeof(char*));
+    memcpy(pos+4, &column, sizeof(char*));
   }
   else if (recinfo[2].type == FIELD_VARCHAR)
   {
     /* Second field is longer than 10 characters */
     uint pack_length= HA_VARCHAR_PACKLENGTH(recinfo[1].length-1);
     uint length= pack_length == 1 ? (uint) *(uchar*) pos : uint2korr(pos);
-    bfill(pos+pack_length+length,recinfo[2].length-length-pack_length,'.');
+    memset(pos + pack_length + length, '.',
+           recinfo[2].length - length-pack_length);
     length=recinfo[2].length-pack_length;
     if (pack_length == 1)
       *(uchar*) pos= (uchar) length;
@@ -525,7 +532,7 @@ static void update_record(uchar *record)
   }
   else
   {
-    bfill(pos+recinfo[2].length-10,10,'.');
+    memset(pos + recinfo[2].length - 10, '.', 10);
   }
 }
 
@@ -534,7 +541,7 @@ static struct my_option my_long_options[] =
 {
   {"checksum", 'c', "Undocumented",
    0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
-#ifndef DBUG_OFF
+#ifndef NDEBUG
   {"debug", '#', "Undocumented",
    0, 0, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 #endif
@@ -586,8 +593,8 @@ static struct my_option my_long_options[] =
 
 
 static my_bool
-get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
-	       char *argument __attribute__((unused)))
+get_one_option(int optid, const struct my_option *opt MY_ATTRIBUTE((unused)),
+	       char *argument MY_ATTRIBUTE((unused)))
 {
   switch(optid) {
   case 'a':
@@ -681,3 +688,5 @@ static void usage()
   my_print_help(my_long_options);
   my_print_variables(my_long_options);
 }
+
+#include "mi_extrafunc.h"
