@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2014, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -41,6 +48,7 @@
 #include <mgmapi/mgmapi.h>
 
 #include <NodeBitmask.hpp>
+#include <NdbMutex.h>
 
 // A transporter is always in an IOState.
 // NoHalt is used initially and as long as it is no restrictions on
@@ -61,7 +69,6 @@ static const char *performStateString[] =
 
 class Transporter;
 class TCP_Transporter;
-class SCI_Transporter;
 class SHM_Transporter;
 
 class TransporterRegistry;
@@ -87,7 +94,7 @@ public:
  * TransporterReceiveData
  *
  *   State for pollReceive/performReceive
- *   Moved into own class to enable multi receive threads
+ *   Moved into own class to enable multiple receive threads
  */
 struct TransporterReceiveData
 {
@@ -100,7 +107,7 @@ struct TransporterReceiveData
    * Add a transporter to epoll_set
    *   does nothing if epoll not active
    */
-  bool epoll_add(TCP_Transporter*);
+  bool epoll_add(Transporter*);
 
   /**
    * Bitmask of transporters currently handled by this instance
@@ -131,10 +138,28 @@ struct TransporterReceiveData
 
   /**
    * Bitmask of transporters having received corrupted or unsupported
-   * message.  No more unpacking and delivery of messages allowed.
+   * message. No more unpacking and delivery of messages allowed.
    */
   NodeBitmask m_bad_data_transporters;
 
+  /**
+   * Last node received from if unable to complete all transporters
+   * in previous ::performReceive(). Next ::performReceive will
+   * resume from first transporter after this.
+   */
+  Uint32 m_last_nodeId;
+
+  /**
+   * Spintime calculated as maximum of currently connected transporters.
+   * Only applies to shared memory transporters.
+   */
+  Uint32 m_spintime;
+
+  /**
+   * Total spintime
+   */
+  Uint32 m_total_spintime;
+
 #if defined(HAVE_EPOLL_CREATE)
   int m_epoll_fd;
   struct epoll_event *m_epoll_events;
@@ -153,7 +178,8 @@ struct TransporterReceiveData
  * @class TransporterRegistry
  * @brief ...
  */
-class TransporterRegistry : private TransporterSendBufferHandle {
+class TransporterRegistry 
+{
   friend class SHM_Transporter;
   friend class SHM_Writer;
   friend class Transporter;
@@ -164,7 +190,6 @@ public:
   */
   TransporterRegistry(TransporterCallback *callback,
                       TransporterReceiveHandle * receiveHandle,
-                      bool use_default_send_buffer = true,
 		      unsigned maxTransporters = MAX_NTRANSPORTERS);
 
   /**
@@ -184,12 +209,26 @@ public:
   bool init(TransporterReceiveHandle&);
 
   /**
-     Handle the handshaking with a new client connection
-     on the server port.
-     NOTE! Connection should be closed if function
+     Perform handshaking of a client connection to accept it
+     as transporter.
+
+     @note Connection should be closed by caller if function
      returns false
+
+     @param sockfd           the socket to handshake
+     @param mgs              error message describing why handshake failed,
+                             to be filled in when function return
+     @param close_with_reset allows the function to indicate to the caller
+                             how the socket should be closed when function
+                             returns false
+     @param log_failure      whether a failure to connect is log-worthy
+
+     @returns false on failure and true on success
   */
-  bool connect_server(NDB_SOCKET_TYPE sockfd, BaseString& errormsg) const;
+  bool connect_server(NDB_SOCKET_TYPE sockfd,
+                      BaseString& msg,
+                      bool& close_with_reset,
+                      bool& log_failure) const;
 
   bool connect_client(NdbMgmHandle *h);
 
@@ -197,7 +236,8 @@ public:
    * Given a SocketClient, creates a NdbMgmHandle, turns it into a transporter
    * and returns the socket.
    */
-  NDB_SOCKET_TYPE connect_ndb_mgmd(SocketClient *sc);
+  NDB_SOCKET_TYPE connect_ndb_mgmd(const char* server_name,
+                                   unsigned short server_port);
 
   /**
    * Given a connected NdbMgmHandle, turns it into a transporter
@@ -223,6 +263,17 @@ private:
    */
   void disconnectAll();
 
+  /**
+   * Reset awake state on shared memory transporters before sleep.
+   */
+  int reset_shm_awake_state(TransporterReceiveHandle& recvdata,
+                            bool& sleep_state_set);
+
+  /**
+   * Set awake state on shared memory transporters after sleep.
+   */
+  void set_shm_awake_state(TransporterReceiveHandle& recvdata);
+
 public:
 
   /**
@@ -267,7 +318,9 @@ public:
    */
   void do_connect(NodeId node_id);
   void do_disconnect(NodeId node_id, int errnum = 0);
-  bool is_connected(NodeId node_id) { return performStates[node_id] == CONNECTED; };
+  bool is_connected(NodeId node_id) const {
+    return performStates[node_id] == CONNECTED;
+  };
 private:
   void report_connect(TransporterReceiveHandle&, NodeId node_id);
   void report_disconnect(TransporterReceiveHandle&, NodeId node_id, int errnum);
@@ -279,14 +332,13 @@ private:
                     size_t sizeOfData,
                     NodeId remoteNodeId,
                     IOState state,
-                    TransporterError errorCode,
-                    Uint32 nextMsgOffset);
+                    TransporterError errorCode);
 public:
   
   /**
    * Get and set methods for IOState
    */
-  IOState ioState(NodeId nodeId);
+  IOState ioState(NodeId nodeId) const;
   void setIOState(NodeId nodeId, IOState state);
 
   /**
@@ -304,7 +356,6 @@ private:
 private:
 
   bool createTCPTransporter(TransporterConfiguration * config);
-  bool createSCITransporter(TransporterConfiguration * config);
   bool createSHMTransporter(TransporterConfiguration * config);
 
 public:
@@ -317,19 +368,6 @@ public:
    */
   bool configureTransporter(TransporterConfiguration * config);
 
-  /**
-   * Allocate send buffer for default send buffer handling.
-   *
-   * Upper layer that implements their own TransporterSendBufferHandle do not
-   * use this, instead they manage their own send buffers.
-   *
-   * Argument is the value of config parameter TotalSendBufferMemory. If 0,
-   * a default will be used of sum(max send buffer) over all transporters.
-   * The second is the config parameter ExtraSendBufferMemory
-   */
-  void allocate_send_buffers(Uint64 total_send_buffer,
-                             Uint64 extra_send_buffer);
-
   /**
    * Get sum of max send buffer over all transporters, to be used as a default
    * for allocate_send_buffers eg.
@@ -337,9 +375,10 @@ public:
    * Must be called after creating all transporters for returned value to be
    * correct.
    */
-  Uint64 get_total_max_send_buffer() { return m_total_max_send_buffer; }
-
-  bool get_using_default_send_buffer() const{ return m_use_default_send_buffer;}
+  Uint64 get_total_max_send_buffer() {
+    assert(m_total_max_send_buffer > 0);
+    return m_total_max_send_buffer;
+  } 
 
   /**
    * Get transporter's connect count
@@ -375,54 +414,46 @@ public:
    *
    * When IOState is HaltOutput or HaltIO do not send or insert any 
    * signals in the SendBuffer, unless it is intended for the remote 
-   * CMVMI block (blockno 252)
+   * QMGR block (blockno 252)
    * Perform prepareSend on the transporter. 
    *
    * NOTE signalHeader->xxxBlockRef should contain block numbers and 
    *                                not references
    */
+
+private:
+  template <typename AnySectionArg>
+  SendStatus prepareSendTemplate(
+                         TransporterSendBufferHandle *sendHandle,
+                         const SignalHeader *signalHeader,
+                         Uint8 prio,
+                         const Uint32 *signalData,
+                         NodeId nodeId,
+                         AnySectionArg section);
+
+
+public:
   SendStatus prepareSend(TransporterSendBufferHandle *sendHandle,
-                         const SignalHeader * const signalHeader, Uint8 prio,
-			 const Uint32 * const signalData,
-			 NodeId nodeId, 
-			 const LinearSectionPtr ptr[3]);
+                         const SignalHeader *signalHeader,
+                         Uint8 prio,
+                         const Uint32 *signalData,
+                         NodeId nodeId,
+                         const LinearSectionPtr ptr[3]);
 
   SendStatus prepareSend(TransporterSendBufferHandle *sendHandle,
-                         const SignalHeader * const signalHeader, Uint8 prio,
-			 const Uint32 * const signalData,
-			 NodeId nodeId, 
-			 class SectionSegmentPool & pool,
-			 const SegmentedSectionPtr ptr[3]);
+                         const SignalHeader *signalHeader,
+                         Uint8 prio,
+                         const Uint32 *signalData,
+                         NodeId nodeId,
+                         class SectionSegmentPool & pool,
+                         const SegmentedSectionPtr ptr[3]);
+
   SendStatus prepareSend(TransporterSendBufferHandle *sendHandle,
-                         const SignalHeader * const signalHeader, Uint8 prio,
-                         const Uint32 * const signalData,
+                         const SignalHeader *signalHeader,
+                         Uint8 prio,
+                         const Uint32 *signalData,
                          NodeId nodeId,
                          const GenericSectionPtr ptr[3]);
-  /**
-   * Backwards compatiple methods with default send buffer handling.
-   */
-  SendStatus prepareSend(const SignalHeader * const signalHeader, Uint8 prio,
-			 const Uint32 * const signalData,
-			 NodeId nodeId,
-			 const LinearSectionPtr ptr[3])
-  {
-    return prepareSend(this, signalHeader, prio, signalData, nodeId, ptr);
-  }
-  SendStatus prepareSend(const SignalHeader * const signalHeader, Uint8 prio,
-			 const Uint32 * const signalData,
-			 NodeId nodeId,
-			 class SectionSegmentPool & pool,
-			 const SegmentedSectionPtr ptr[3])
-  {
-    return prepareSend(this, signalHeader, prio, signalData, nodeId, pool, ptr);
-  }
-  SendStatus prepareSend(const SignalHeader * const signalHeader, Uint8 prio,
-                         const Uint32 * const signalData,
-                         NodeId nodeId,
-                         const GenericSectionPtr ptr[3])
-  {
-    return prepareSend(this, signalHeader, prio, signalData, nodeId, ptr);
-  }
   
   /**
    * external_IO
@@ -432,15 +463,8 @@ public:
    */
   void external_IO(Uint32 timeOutMillis);
 
-  bool performSend(NodeId nodeId);
+  bool performSend(NodeId nodeId, bool need_wakeup = true);
   void performSend();
-
-  /**
-   * Force sending if more than or equal to sendLimit
-   * number have asked for send. Returns 0 if not sending
-   * and 1 if sending.
-   */
-  int forceSendCheck(int sendLimit);
   
 #ifdef DEBUG_TRANSPORTER
   void printState();
@@ -455,16 +479,18 @@ public:
   Vector<Transporter_interface> m_transporter_interface;
   void add_transporter_interface(NodeId remoteNodeId, const char *interf,
 		  		 int s_port);	// signed port. <0 is dynamic
-  Transporter* get_transporter(NodeId nodeId);
+
+  int get_transporter_count() const;
+  Transporter* get_transporter(NodeId nodeId) const;
+  bool is_shm_transporter(NodeId nodeId);
   struct in_addr get_connect_address(NodeId node_id) const;
 
   Uint64 get_bytes_sent(NodeId nodeId) const;
   Uint64 get_bytes_received(NodeId nodeId) const;
-protected:
   
 private:
-  TransporterCallback *callbackObj;
-  TransporterReceiveHandle * receiveHandle;
+  TransporterCallback *const callbackObj;
+  TransporterReceiveHandle *const receiveHandle;
 
   NdbMgmHandle m_mgm_handle;
 
@@ -474,24 +500,25 @@ private:
   int sendCounter;
   NodeId localNodeId;
   unsigned maxTransporters;
-  int nTransporters;
-  int nTCPTransporters;
-  int nSCITransporters;
-  int nSHMTransporters;
+  Uint32 nTransporters;
+  Uint32 nTCPTransporters;
+  Uint32 nSHMTransporters;
 
 #ifdef ERROR_INSERT
   Bitmask<MAX_NTRANSPORTERS/32> m_blocked;
   Bitmask<MAX_NTRANSPORTERS/32> m_blocked_disconnected;
   int m_disconnect_errors[MAX_NTRANSPORTERS];
 
+  Bitmask<MAX_NTRANSPORTERS/32> m_sendBlocked;
+
   Uint32 m_mixology_level;
 #endif
 
   /**
    * Arrays holding all transporters in the order they are created
    */
+  Transporter**     allTransporters;
   TCP_Transporter** theTCPTransporters;
-  SCI_Transporter** theSCITransporters;
   SHM_Transporter** theSHMTransporters;
   
   /**
@@ -548,15 +575,20 @@ private:
                 Uint32 * readPtr,
                 Uint32 bufferSize,
                 NodeId remoteNodeId,
-                IOState state);
+                IOState state,
+		bool & stopReceiving);
 
   Uint32 * unpack(TransporterReceiveHandle&,
                   Uint32 * readPtr,
                   Uint32 * eodPtr,
+                  Uint32 * endPtr,
                   NodeId remoteNodeId,
-                  IOState state);
+                  IOState state,
+		  bool & stopReceiving);
 
-  static Uint32 unpack_length_words(const Uint32 *readPtr, Uint32 maxWords);
+  static Uint32 unpack_length_words(const Uint32 *readPtr,
+                                    Uint32 maxWords,
+                                    bool extra_signal);
   /** 
    * Disconnect the transporter and remove it from 
    * theTransporters array. Do not allow any holes 
@@ -566,11 +598,15 @@ private:
   void removeTransporter(NodeId nodeId);
 
   Uint32 poll_TCP(Uint32 timeOutMillis, TransporterReceiveHandle&);
-  Uint32 poll_SCI(Uint32 timeOutMillis, TransporterReceiveHandle&);
-  Uint32 poll_SHM(Uint32 timeOutMillis, TransporterReceiveHandle&);
+  Uint32 poll_SHM(TransporterReceiveHandle&, bool &any_connected);
+  Uint32 poll_SHM(TransporterReceiveHandle&,
+                  NDB_TICKS start_time,
+                  Uint32 micros_to_poll);
+  Uint32 check_TCP(TransporterReceiveHandle&, Uint32 timeoutMillis);
+  Uint32 spin_check_transporters(TransporterReceiveHandle&);
 
   int m_shm_own_pid;
-  int m_transp_count;
+  Uint32 m_transp_count;
 
 public:
   bool setup_wakeup_socket(TransporterReceiveHandle&);
@@ -585,70 +621,17 @@ private:
   NDB_SOCKET_TYPE m_extra_wakeup_sockets[2];
   void consume_extra_sockets();
 
-
   Uint32 *getWritePtr(TransporterSendBufferHandle *handle,
                       NodeId node, Uint32 lenBytes, Uint32 prio);
   void updateWritePtr(TransporterSendBufferHandle *handle,
                       NodeId node, Uint32 lenBytes, Uint32 prio);
 
-  /**
-   * TransporterSendBufferHandle implementation.
-   *
-   * Used for default send buffer handling, when the upper layer does not
-   * want to do special buffer handling itself.
-   */
-  virtual Uint32 *getWritePtr(NodeId node, Uint32 lenBytes, Uint32 prio,
-                              Uint32 max_use);
-  virtual Uint32 updateWritePtr(NodeId node, Uint32 lenBytes, Uint32 prio);
-  virtual bool forceSend(NodeId node);
-
-
+public:
   /* Various internal */
   void inc_overload_count(Uint32 nodeId);
   void inc_slowdown_count(Uint32 nodeId);
-private:
-  /* Send buffer pages. */
-  struct SendBufferPage {
-    /* This is the number of words that will fit in one page of send buffer. */
-    static const Uint32 PGSIZE = 32768;
-    static Uint32 max_data_bytes()
-    {
-      return PGSIZE - offsetof(SendBufferPage, m_data);
-    }
-
-    /* Send buffer for one transporter is kept in a single-linked list. */
-    struct SendBufferPage *m_next;
-
-    /* Bytes of send data available in this page. */
-    Uint16 m_bytes;
-    /* Start of unsent data */
-    Uint16 m_start;
-
-    /* Data; real size is to the end of one page. */
-    char m_data[2];
-  };
-
-  /* Send buffer for one transporter. */
-  struct SendBuffer {
-    /* Total size of data in buffer, from m_offset_start_data to end. */
-    Uint32 m_used_bytes;
-    /* Linked list of active buffer pages with first and last pointer. */
-    SendBufferPage *m_first_page;
-    SendBufferPage *m_last_page;
-  };
-
-  SendBufferPage *alloc_page();
-  void release_page(SendBufferPage *page);
 
 private:
-  /* True if we are using the default send buffer implementation. */
-  bool m_use_default_send_buffer;
-  /* Send buffers. */
-  SendBuffer *m_send_buffers;
-  /* Linked list of free pages. */
-  SendBufferPage *m_page_freelist;
-  /* Original block of memory for pages (so we can free it at exit). */
-  unsigned char *m_send_buffer_memory;
   /**
    * Sum of max transporter memory for each transporter.
    * Used to compute default send buffer size.
@@ -656,14 +639,6 @@ private:
   Uint64 m_total_max_send_buffer;
 
 public:
-  Uint32 get_bytes_to_send_iovec(NodeId node, struct iovec *dst, Uint32 max);
-  Uint32 bytes_sent(NodeId node, Uint32 bytes);
-  bool has_data_to_send(NodeId node);
-
-  void reset_send_buffer(NodeId node, bool should_be_empty);
-
-  void print_transporters(const char* where, NdbOut& out = ndbout);
-
   /**
    * Receiving
    */
@@ -685,12 +660,27 @@ public:
     assert(receiveHandle != 0);
     update_connections(* receiveHandle);
   }
+  inline Uint32 get_total_spintime()
+  {
+    assert(receiveHandle != 0);
+    return receiveHandle->m_total_spintime;
+  }
+  inline void reset_total_spintime()
+  {
+    assert(receiveHandle != 0);
+    receiveHandle->m_total_spintime = 0;
+  }
 
 #ifdef ERROR_INSERT
   /* Utils for testing latency issues */
   bool isBlocked(NodeId nodeId);
   void blockReceive(TransporterReceiveHandle&, NodeId nodeId);
   void unblockReceive(TransporterReceiveHandle&, NodeId nodeId);
+  bool isSendBlocked(NodeId nodeId) const;
+  void blockSend(TransporterReceiveHandle& recvdata,
+                 NodeId nodeId);
+  void unblockSend(TransporterReceiveHandle& recvdata,
+                   NodeId nodeId);
 
   /* Testing interleaving of signal processing */
   Uint32 getMixologyLevel() const;
@@ -826,4 +816,15 @@ TransporterRegistry::backoff_update_and_check_time_for_connect(NodeId nodeId)
   return (connectingTime[nodeId] % attempt_moments[attempt_moments_count - 1] == 0);
 }
 
+/**
+ * A function used to calculate a send buffer level given the size of the node
+ * send buffer and the total send buffer size for all nodes and the total send
+ * buffer used for all nodes. There is also a thread parameter that specifies
+ * the number of threads used (this is 0 except for ndbmtd).
+ */
+void calculate_send_buffer_level(Uint64 node_send_buffer_size,
+                                 Uint64 total_send_buffer_size,
+                                 Uint64 total_used_send_buffer_size,
+                                 Uint32 num_threads,
+                                 SB_LevelType &level);
 #endif // Define of TransporterRegistry_H
