@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -79,12 +86,7 @@ public:
    */
   static void clear(unsigned size, Uint32 data[]);
 
-  /**
-   * clear bit from <em>start</em> to <em>last</em>
-   */
-  static void clear_range(unsigned size, Uint32 data[], unsigned start, unsigned last);
-
-  static Uint32 getWord(unsigned size, Uint32 data[], unsigned word_pos);
+  static Uint32 getWord(unsigned size, const Uint32 data[], unsigned word_pos);
   static void setWord(unsigned size, Uint32 data[],
                       unsigned word_pos, Uint32 new_word);
   /**
@@ -190,12 +192,12 @@ public:
   /**
    * contains - Check if all bits set in data2 are set in data
    */
-  static bool contains(unsigned size, Uint32 data[], const Uint32 data2[]);
+  static bool contains(unsigned size, const Uint32 data[], const Uint32 data2[]);
 
   /**
    * overlaps - Check if any bit set in data is set in data2
    */
-  static bool overlaps(unsigned size, Uint32 data[], const Uint32 data2[]);
+  static bool overlaps(unsigned size, const Uint32 data[], const Uint32 data2[]);
 
   /**
    * getField - Get bitfield at given position and length (max 32 bits)
@@ -292,6 +294,11 @@ inline void
 BitmaskImpl::setRange(unsigned size, Uint32 data[],
                       unsigned start, unsigned len)
 {
+  if (len == 0)
+  {
+    return;
+  }
+
   Uint32 last = start + len - 1;
   Uint32 *ptr = data + (start >> 5);
   Uint32 *end =  data + (last >> 5);
@@ -312,7 +319,7 @@ BitmaskImpl::setRange(unsigned size, Uint32 data[],
     tmp_word = ~(Uint32)0;
   }
 
-  tmp_word &= ~(~(Uint32)0 << (last & 31));
+  tmp_word &= ~(~(Uint32)1 << (last & 31));
   
   * ptr |= tmp_word;
 }
@@ -340,37 +347,9 @@ BitmaskImpl::clear(unsigned size, Uint32 data[])
   }
 }
 
-inline void
-BitmaskImpl::clear_range(unsigned size, Uint32 data[], 
-			 unsigned start, unsigned last)
-{
-  Uint32 *ptr = data + (start >> 5);
-  Uint32 *end =  data + (last >> 5);
-  assert(start <= last);
-  assert(last < (size << 5));
-  
-  Uint32 tmp_word = ~(Uint32)0 << (start & 31);
-
-  if (ptr < end)
-  {
-    * ptr ++ &= ~tmp_word;
-    
-    for(; ptr < end; )
-    {
-      * ptr ++ = 0;
-    }
-    
-    tmp_word = ~(Uint32)0;
-  }
-
-  tmp_word &= ~(~(Uint32)0 << (last & 31));
-
-  * ptr &= ~tmp_word;
-}
-
 inline
 Uint32
-BitmaskImpl::getWord(unsigned size, Uint32 data[], unsigned word_pos)
+BitmaskImpl::getWord(unsigned size, const Uint32 data[], unsigned word_pos)
 {
   return data[word_pos];
 }
@@ -732,7 +711,7 @@ BitmaskImpl::bitNOT(unsigned size, Uint32 data[])
 }
 
 inline bool
-BitmaskImpl::contains(unsigned size, Uint32 data[], const Uint32 data2[])
+BitmaskImpl::contains(unsigned size, const Uint32 data[], const Uint32 data2[])
 {
   for (unsigned int i = 0; i < size; i++)
     if ((data[i] & data2[i]) != data2[i])
@@ -741,7 +720,7 @@ BitmaskImpl::contains(unsigned size, Uint32 data[], const Uint32 data2[])
 }
 
 inline bool
-BitmaskImpl::overlaps(unsigned size, Uint32 data[], const Uint32 data2[])
+BitmaskImpl::overlaps(unsigned size, const Uint32 data[], const Uint32 data2[])
 {
   for (unsigned int i = 0; i < size; i++)
     if ((data[i] & data2[i]) != 0)
@@ -929,7 +908,7 @@ public:
   /**
    * Get and set words of bits
    */
-  Uint32 getWord(unsigned word_pos);
+  Uint32 getWord(unsigned word_pos) const;
   void setWord(unsigned word_pos, Uint32 new_word);
   
   /**
@@ -1025,14 +1004,14 @@ public:
   /**
    * contains - Check if all bits set in data2 (that) are also set in data (this)
    */
-  static bool contains(Uint32 data[], const Uint32 data2[]);
-  bool contains(BitmaskPOD<size> that);
+  static bool contains(const Uint32 data[], const Uint32 data2[]);
+  bool contains(BitmaskPOD<size> that) const;
 
   /**
    * overlaps - Check if any bit set in this BitmaskPOD (data) is also set in that (data2)
    */
-  static bool overlaps(Uint32 data[], const Uint32 data2[]);
-  bool overlaps(BitmaskPOD<size> that);
+  static bool overlaps(const Uint32 data[], const Uint32 data2[]);
+  bool overlaps(BitmaskPOD<size> that) const;
 
   /**
    * getText - Return as hex-digits (only for debug routines).
@@ -1201,7 +1180,7 @@ BitmaskPOD<size>::clear()
 
 template <unsigned size>
 inline Uint32
-BitmaskPOD<size>::getWord(unsigned word_pos)
+BitmaskPOD<size>::getWord(unsigned word_pos) const
 {
   return BitmaskImpl::getWord(size, rep.data, word_pos);
 }
@@ -1431,28 +1410,28 @@ BitmaskPOD<size>::getText(char* buf) const
 
 template <unsigned size>
 inline bool
-BitmaskPOD<size>::contains(Uint32 data[], const Uint32 data2[])
+BitmaskPOD<size>::contains(const Uint32 data[], const Uint32 data2[])
 {
   return BitmaskImpl::contains(size, data, data2);
 }
 
 template <unsigned size>
 inline bool
-BitmaskPOD<size>::contains(BitmaskPOD<size> that)
+BitmaskPOD<size>::contains(BitmaskPOD<size> that) const
 {
   return BitmaskPOD<size>::contains(this->rep.data, that.rep.data);
 }
 
 template <unsigned size>
 inline bool
-BitmaskPOD<size>::overlaps(Uint32 data[], const Uint32 data2[])
+BitmaskPOD<size>::overlaps(const Uint32 data[], const Uint32 data2[])
 {
   return BitmaskImpl::overlaps(size, data, data2);
 }
 
 template <unsigned size>
 inline bool
-BitmaskPOD<size>::overlaps(BitmaskPOD<size> that)
+BitmaskPOD<size>::overlaps(BitmaskPOD<size> that) const
 {
   return BitmaskPOD<size>::overlaps(this->rep.data, that.rep.data);
 }
@@ -1477,6 +1456,7 @@ template <unsigned size>
 class Bitmask : public BitmaskPOD<size> {
 public:
   Bitmask() { this->clear();}
+  Bitmask(bool v) { (void)v; }
 
   template<unsigned sz2> Bitmask& operator=(const Bitmask<sz2>& src){
     if (size >= sz2)
