@@ -37,7 +37,11 @@ Dbspj::Dbspj(Block_context& ctx, Uint32 instanceNumber):
   SimulatedBlock(DBSPJ, ctx, instanceNumber),
   m_scan_request_hash(m_request_pool),
   m_lookup_request_hash(m_request_pool),
-  m_tableRecord(NULL), c_tabrecFilesize(0)
+  m_treenode_hash(m_treenode_pool),
+  m_scanfraghandle_hash(m_scanfraghandle_pool),
+  m_tableRecord(NULL),
+  c_tabrecFilesize(0),
+  m_load_balancer_location(0)
 {
   BLOCK_CONSTRUCTOR(Dbspj);
 
