@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -38,6 +45,9 @@
 #include <signaldata/CreateTab.hpp>
 #include <signaldata/LqhFrag.hpp>
 
+#define JAM_FILE_ID 377
+
+
 void
 Dbtux::execCREATE_TAB_REQ(Signal* signal)
 {
@@ -184,6 +194,7 @@ Dbtux::execTUX_ADD_ATTRREQ(Signal* signal)
 #endif
     if (csNumber != 0) {
       unsigned err;
+      ndbrequire(csNumber <= NDB_ARRAY_SIZE(all_charsets));
       CHARSET_INFO *cs = all_charsets[csNumber];
       ndbrequire(cs != 0);
       if ((err = NdbSqlUtil::check_column_for_ordered_index(typeId, cs))) {
@@ -204,25 +215,32 @@ Dbtux::execTUX_ADD_ATTRREQ(Signal* signal)
       // compute min prefix
       const KeySpec& keySpec = indexPtr.p->m_keySpec;
       unsigned attrs = 0;
-      unsigned bytes = keySpec.get_nullmask_len(false);
+      unsigned bytes = 0;
       unsigned maxAttrs = indexPtr.p->m_numAttrs;
 #ifdef VM_TRACE
+#ifdef NDB_USE_GET_ENV
       {
         const char* p = NdbEnv_GetEnv("MAX_TTREE_PREF_ATTRS", (char*)0, 0);
         if (p != 0 && p[0] != 0 && maxAttrs > (unsigned)atoi(p))
           maxAttrs = atoi(p);
       }
 #endif
-      while (attrs < maxAttrs) {
+#endif
+      while (attrs < maxAttrs)
+      {
+        /**
+         * Prefix is now saved as a normal Attrinfo data stream.
+         * This means that each column uses 4 bytes Attrinfo header
+         * the data is aligned on a word boundary.
+         */
         const KeyType& keyType = keySpec.get_type(attrs);
-        const unsigned newbytes = bytes + keyType.get_byte_size();
+        const unsigned word_size = (keyType.get_byte_size() + 3) / 4;
+        const unsigned newbytes = bytes + ((word_size + 1) * 4);
         if (newbytes > (MAX_TTREE_PREF_SIZE << 2))
           break;
         attrs++;
         bytes = newbytes;
       }
-      if (attrs == 0)
-        bytes = 0;
       indexPtr.p->m_prefAttrs = attrs;
       indexPtr.p->m_prefBytes = bytes;
       // fragment is defined
@@ -311,7 +329,7 @@ Dbtux::execTUXFRAGREQ(Signal* signal)
     fragPtr.p->m_tupTableFragPtrI = req->tupTableFragPtrI;
     fragPtr.p->m_accTableFragPtrI = req->accTableFragPtrI;
     // add the fragment to the index
-    Uint32 fragNo = indexPtr.p->m_numFrags;
+    const Uint32 fragNo = indexPtr.p->m_numFrags;
     indexPtr.p->m_fragId[indexPtr.p->m_numFrags] = req->fragId;
     indexPtr.p->m_fragPtrI[indexPtr.p->m_numFrags] = fragPtr.i;
     indexPtr.p->m_numFrags++;
@@ -652,8 +670,8 @@ Dbtux::execDROP_FRAG_REQ(Signal* signal)
 
   IndexPtr indexPtr;
   c_indexPool.getPtr(indexPtr, req->tableId);
-  Uint32 i = 0;
-  for (i = 0; i < indexPtr.p->m_numFrags; i++)
+
+  for (Uint32 i = 0; i < indexPtr.p->m_numFrags; i++)
   {
     jam();
     if (indexPtr.p->m_fragId[i] == req->fragId)
@@ -666,6 +684,9 @@ Dbtux::execDROP_FRAG_REQ(Signal* signal)
       for (i++; i < indexPtr.p->m_numFrags; i++)
       {
         jam();
+        /* Check array bounds to silence gcc 4.8.2 false warning bug */
+        ndbrequire(i < NDB_ARRAY_SIZE(indexPtr.p->m_fragPtrI) &&
+                   i < NDB_ARRAY_SIZE(indexPtr.p->m_fragId));
         indexPtr.p->m_fragPtrI[i-1] = indexPtr.p->m_fragPtrI[i];
         indexPtr.p->m_fragId[i-1] = indexPtr.p->m_fragId[i];
       }
