@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2009, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,6 +22,8 @@
 
 #include <ndb_global.h>
 
+#include <algorithm>
+
 #include <NdbEnv.h>
 #include <NdbConfig.h>
 #include <NdbSleep.h>
@@ -46,6 +48,8 @@
 #endif
 
 #include <EventLogger.hpp>
+#include <OutputStream.hpp>
+#include <LogBuffer.hpp>
 
 #define JAM_FILE_ID 484
 
@@ -160,7 +164,7 @@ parse_key_value_before_filespecs(const char *src,
   for (unsigned i = 0; i < list.size(); i++)
   {
     list[i].trim();
-    if (strncasecmp(list[i].c_str(), key, keylen) == 0)
+    if (native_strncasecmp(list[i].c_str(), key, keylen) == 0)
     {
       // key found, save its value
       value = parse_size(list[i].c_str() + keylen);
@@ -251,7 +255,7 @@ compute_acc_32kpages(const ndb_mgm_configuration_iterator * p)
  * overallocated at any size as long as there is still memory
  * remaining.
  *
- * RG_DISK_OPERATIONS:
+ * RG_TRANSACTION_MEMORY:
  * This is a resource that is either set to zero size but can be overallocated
  * without limit. If a log file group is allocated based on the config, then
  * the size of the UNDO log buffer is used to set the size of this resource.
@@ -411,9 +415,11 @@ init_global_memory_manager(EmulatorData &ed, Uint32 *watchCounter)
     rl.m_min = sbpages;
     /**
      * allow over allocation (from SharedGlobalMemory) of up to 25% of
-     *   totally allocated SendBuffer
+     *   totally allocated SendBuffer, at most 25% of SharedGlobalMemory.
      */
-    rl.m_max = sbpages + (sbpages * 25) / 100;
+    const Uint32 sb_max_shared_pages = 25 * MIN(sbpages, shared_pages) / 100;
+    require(sbpages + sb_max_shared_pages > 0);
+    rl.m_max = sbpages + sb_max_shared_pages;
     rl.m_resource_id = RG_TRANSPORTER_BUFFERS;
     ed.m_mem_manager->set_resource_limit(rl);
   }
@@ -449,7 +455,7 @@ init_global_memory_manager(EmulatorData &ed, Uint32 *watchCounter)
     ed.m_mem_manager->set_resource_limit(rl);
   }
 
-  Uint32 undopages = 0;
+  Uint32 transmem = 0;
   {
     /**
      * Request extra undo buffer memory to be allocated when
@@ -477,31 +483,23 @@ init_global_memory_manager(EmulatorData &ed, Uint32 *watchCounter)
                                          "undo_buffer_size=",
                                          undo_buffer_size);
 
-        undopages = Uint32(undo_buffer_size / GLOBAL_PAGE_SIZE);
+        Uint32 undopages = Uint32(undo_buffer_size / GLOBAL_PAGE_SIZE);
         g_eventLogger->info("reserving %u extra pages for undo buffer memory",
                             undopages);
+        transmem = undopages;
         Resource_limit rl;
-        rl.m_min = undopages;
+        rl.m_min = transmem;
         rl.m_max = 0;
-        rl.m_resource_id = RG_DISK_OPERATIONS;
+        rl.m_resource_id = RG_TRANSACTION_MEMORY;
         ed.m_mem_manager->set_resource_limit(rl);
       }
     }
   }
 
   Uint32 sum = shared_pages + tupmem + filepages + jbpages + sbpages +
-    pgman_pages + stpages + undopages;
+    pgman_pages + stpages + transmem;
 
-  if (sum)
-  {
-    Resource_limit rl;
-    rl.m_min = 0;
-    rl.m_max = sum;
-    rl.m_resource_id = 0;
-    ed.m_mem_manager->set_resource_limit(rl);
-  }
-
-  if (!ed.m_mem_manager->init(watchCounter))
+  if (!ed.m_mem_manager->init(watchCounter, sum))
   {
     struct ndb_mgm_param_info dm;
     struct ndb_mgm_param_info sga;
@@ -644,10 +642,10 @@ void
 handler_error(int signum){
   // only let one thread run shutdown
   static bool handling_error = false;
-  static pthread_t thread_id; // Valid when handling_error is true
+  static my_thread_t thread_id; // Valid when handling_error is true
 
   if (handling_error &&
-      pthread_equal(thread_id, pthread_self()))
+      my_thread_equal(thread_id, my_thread_self()))
   {
     // Shutdown thread received signal
 #ifndef NDB_WIN32
@@ -661,7 +659,7 @@ handler_error(int signum){
     while(true)
       NdbSleep_MilliSleep(10);
 
-  thread_id = pthread_self();
+  thread_id = my_thread_self();
   handling_error = true;
 
   g_eventLogger->info("Received signal %d. Running error handler.", signum);
@@ -692,7 +690,11 @@ catchsigs(bool foreground){
 #elif defined SIGINFO
     SIGINFO,
 #endif
+#ifdef _WIN32
+    SIGTERM,
+#else
     SIGQUIT,
+#endif
     SIGTERM,
 #ifdef SIGTSTP
     SIGTSTP,
@@ -728,7 +730,11 @@ catchsigs(bool foreground){
   };
 
   static const int signals_ignore[] = {
+#ifdef _WIN32
+    SIGINT
+#else
     SIGPIPE
+#endif
   };
 
   size_t i;
@@ -760,13 +766,86 @@ DWORD WINAPI shutdown_thread(LPVOID)
 }
 #endif
 
+struct ThreadData
+{
+  FILE* f;
+  LogBuffer* logBuf;
+  bool stop;
+};
+
+/**
+ * This function/thread is responsible for getting
+ * bytes from the log buffer and writing them
+ * to the log file.
+ */
+
+void* async_log_func(void* args)
+{
+  ThreadData* data = (ThreadData*)args;
+  FILE* f = data->f;
+  LogBuffer* logBuf = data->logBuf;
+  const size_t get_bytes = 512;
+  char buf[get_bytes + 1];
+  size_t bytes;
+  int part_bytes = 0, bytes_printed = 0;
+
+  while(!data->stop)
+  {
+    part_bytes = 0;
+    bytes_printed = 0;
+
+    if((bytes = logBuf->get(buf, get_bytes)))
+    {
+      fwrite(buf, bytes, 1, f);
+      fflush(f);
+    }
+  }
+
+  while((bytes = logBuf->get(buf, get_bytes, 1)))// flush remaining logs
+  {
+    fwrite(buf, bytes, 1, f);
+    fflush(f);
+  }
+
+  // print lost count in the end, if any
+  size_t lost_count = logBuf->getLostCount();
+  if(lost_count)
+  {
+    fprintf(f, "\n*** %lu BYTES LOST ***\n", (unsigned long)lost_count);
+    fflush(f);
+  }
+
+  return NULL;
+}
 
 void
 ndbd_run(bool foreground, int report_fd,
          const char* connect_str, int force_nodeid, const char* bind_address,
          bool no_start, bool initial, bool initialstart,
-         unsigned allocated_nodeid, int connect_retries, int connect_delay)
+         unsigned allocated_nodeid, int connect_retries, int connect_delay,
+         size_t logbuffer_size)
 {
+  LogBuffer* logBuf = new LogBuffer(logbuffer_size);
+  BufferedOutputStream* ndbouts_bufferedoutputstream = new BufferedOutputStream(logBuf);
+
+  // Make ndbout point to the BufferedOutputStream.
+  NdbOut_ReInit(ndbouts_bufferedoutputstream, ndbouts_bufferedoutputstream);
+
+  struct NdbThread* log_threadvar= NULL;
+  ThreadData thread_args=
+  {
+    stdout,
+    logBuf,
+    false
+  };
+
+  // Create log thread.
+  log_threadvar = NdbThread_Create(async_log_func,
+                       (void**)&thread_args,
+                       0,
+                       (char*)"async_log_thread",
+                       NDB_THREAD_PRIO_MEAN);
+
 #ifdef _WIN32
   {
     char shutdown_event_name[32];
@@ -909,6 +988,8 @@ ndbd_run(bool foreground, int report_fd,
   */
   globalEmulatorData.theThreadConfig->init();
 
+  globalEmulatorData.theConfiguration->addThread(log_threadvar, NdbfsThread);
+
 #ifdef VM_TRACE
   // Initialize signal logger before block constructors
   char *signal_log_name = NdbConfig_SignalLogFileName(globalData.ownId);
@@ -939,7 +1020,7 @@ ndbd_run(bool foreground, int report_fd,
     ndbout_c("Failed to open signal logging file '%s', errno: %d",
              signal_log_name, errno);
   }
-  NdbMem_Free(signal_log_name);
+  free(signal_log_name);
 #endif
 
   /** Create all the blocks used by the run-time environment. */
@@ -1038,8 +1119,21 @@ ndbd_run(bool foreground, int report_fd,
   globalEmulatorData.theConfiguration->removeThread(pWatchdog);
   globalEmulatorData.theConfiguration->removeThread(pTrp);
   globalEmulatorData.theConfiguration->removeThread(pSockServ);
+
   NdbShutdown(0, NST_Normal);
 
+  /**
+   * Stopping the log thread is done at the very end since the
+   * data node logs should be available until complete shutdown.
+   */
+  void* dummy_return_status;
+  thread_args.stop = true;
+  logBuf->stop();
+  NdbThread_WaitFor(log_threadvar, &dummy_return_status);
+  globalEmulatorData.theConfiguration->removeThread(log_threadvar);
+  NdbThread_Destroy(&log_threadvar);
+  delete logBuf;
+  delete ndbouts_bufferedoutputstream;
   ndbd_exit(0);
 }
 
@@ -1154,7 +1248,7 @@ NdbShutdown(int error_code,
      *   cause with ndbmtd, there are locks and nasty stuff
      *   and we don't know which we are holding...
      */
-#if NOT_YET
+#ifdef NOT_YET
 
     /**
      * Stop all transporter connection attempts and accepts
