@@ -50,6 +50,7 @@ template <class T> class Array;
 template <class T>
 class ArrayPool {
 public:
+  typedef T Type;
   typedef void (CallBack)(ArrayPool<T>& pool);
 
   /* 
@@ -122,26 +123,26 @@ public:
   /**
    * Update p value for ptr according to i value 
    */
-  void getPtr(Ptr<T> &);
+  void getPtr(Ptr<T> &) const;
   void getPtr(ConstPtr<T> &) const;
-  void getPtr(Ptr<T> &, bool CrashOnBoundaryError);
+  void getPtr(Ptr<T> &, bool CrashOnBoundaryError) const;
   void getPtr(ConstPtr<T> &, bool CrashOnBoundaryError) const;
-  void getPtrIgnoreAlloc(Ptr<T> &);
+  void getPtrIgnoreAlloc(Ptr<T> &) const;
   
   /**
    * Get pointer for i value
    */
-  T * getPtr(Uint32 i);
+  T * getPtr(Uint32 i) const;
   const T * getConstPtr(Uint32 i) const;
-  T * getPtr(Uint32 i, bool CrashOnBoundaryError);
+  T * getPtr(Uint32 i, bool CrashOnBoundaryError) const;
   const T * getConstPtr(Uint32 i, bool CrashOnBoundaryError) const;
 
   /**
    * Update p & i value for ptr according to <b>i</b> value 
    */
-  void getPtr(Ptr<T> &, Uint32 i);
+  void getPtr(Ptr<T> &, Uint32 i) const;
   void getPtr(ConstPtr<T> &, Uint32 i) const;
-  void getPtr(Ptr<T> &, Uint32 i, bool CrashOnBoundaryError);
+  void getPtr(Ptr<T> &, Uint32 i, bool CrashOnBoundaryError) const;
   void getPtr(ConstPtr<T> &, Uint32 i, bool CrashOnBoundaryError) const;
 
   /**
@@ -192,48 +193,6 @@ public:
   }
 #endif
 
-  /**
-   * Cache+LockFun is used to make thread-local caches for ndbmtd
-   *   I.e each thread has one cache-instance, and can seize/release on this
-   *       wo/ locks
-   */
-  struct Cache
-  {
-    Cache(Uint32 a0 = 512, Uint32 a1 = 256)
-    { m_first_free = RNIL; m_free_cnt = 0; m_alloc_cnt = a0; m_max_free_cnt = a1; }
-    void init_cache(Uint32 a0, Uint32 a1)
-    {
-      m_alloc_cnt = a0;
-      m_max_free_cnt = a1;
-    }
-    Uint32 m_first_free;
-    Uint32 m_free_cnt;
-    Uint32 m_alloc_cnt;
-    Uint32 m_max_free_cnt;
-  };
-
-  struct LockFun
-  {
-    void (* lock)(void);
-    void (* unlock)(void);
-  };
-
-  bool seize(LockFun, Cache&, Ptr<T> &);
-  void release(LockFun, Cache&, Uint32 i);
-  void release(LockFun, Cache&, Ptr<T> &);
-  void releaseList(LockFun, Cache&, Uint32 n, Uint32 first, Uint32 last);
-
-  void setChunkSize(Uint32 sz);
-#ifdef ARRAY_CHUNK_GUARD
-  void checkChunks();
-#endif
-
-protected:
-  void releaseChunk(LockFun, Cache&, Uint32 n);
-
-  bool seizeChunk(Uint32 & n, Ptr<T> &);
-  void releaseChunk(Uint32 n, Uint32 first, Uint32 last);
-
 protected:
   friend class Array<T>;
 
@@ -440,32 +399,6 @@ ArrayPool<T>::setSize(Uint32 noOfElements,
   return false; // not reached
 }
 
-template <class T>
-inline
-void
-ArrayPool<T>::setChunkSize(Uint32 sz)
-{
-  Uint32 i;
-  for (i = 0; i + sz < size; i += sz)
-  {
-    theArray[i].chunkSize = sz;
-    theArray[i].lastChunk = i + sz - 1;
-    theArray[i].nextChunk = i + sz;
-  }
-
-  theArray[i].chunkSize = size - i;
-  theArray[i].lastChunk = size - 1;
-  theArray[i].nextChunk = RNIL;
-
-#ifdef ARRAY_GUARD
-  chunk = true;
-#endif
-
-#ifdef ARRAY_CHUNK_GUARD
-  checkChunks();
-#endif
-}
-
 template <class T>
 inline
 bool
@@ -502,7 +435,8 @@ ArrayPool<T>::set(T* ptr, Uint32 cnt, bool align){
 template <class T>
 inline
 void
-ArrayPool<T>::getPtr(Ptr<T> & ptr){
+ArrayPool<T>::getPtr(Ptr<T> & ptr) const
+{
   Uint32 i = ptr.i;
   if(likely (i < size)){
     ptr.p = &theArray[i];
@@ -525,7 +459,8 @@ ArrayPool<T>::getPtr(Ptr<T> & ptr){
 template <class T>
 inline
 void
-ArrayPool<T>::getPtr(ConstPtr<T> & ptr) const {
+ArrayPool<T>::getPtr(ConstPtr<T> & ptr) const
+{
   Uint32 i = ptr.i;
   if(likely(i < size)){
     ptr.p = &theArray[i];
@@ -548,7 +483,8 @@ ArrayPool<T>::getPtr(ConstPtr<T> & ptr) const {
 template <class T>
 inline
 void
-ArrayPool<T>::getPtr(Ptr<T> & ptr, Uint32 i){
+ArrayPool<T>::getPtr(Ptr<T> & ptr, Uint32 i) const
+{
   ptr.i = i;
   if(likely(i < size)){
     ptr.p = &theArray[i];
@@ -571,7 +507,8 @@ ArrayPool<T>::getPtr(Ptr<T> & ptr, Uint32 i){
 template <class T>
 inline
 void
-ArrayPool<T>::getPtr(ConstPtr<T> & ptr, Uint32 i) const {
+ArrayPool<T>::getPtr(ConstPtr<T> & ptr, Uint32 i) const
+{
   ptr.i = i;
   if(likely(i < size)){
     ptr.p = &theArray[i];
@@ -594,7 +531,8 @@ ArrayPool<T>::getPtr(ConstPtr<T> & ptr, Uint32 i) const {
 template <class T>
 inline
 T * 
-ArrayPool<T>::getPtr(Uint32 i){
+ArrayPool<T>::getPtr(Uint32 i) const
+{
   if(likely(i < size)){
 #ifdef ARRAY_GUARD
     if (theAllocatedBitmask)
@@ -642,7 +580,8 @@ ArrayPool<T>::getConstPtr(Uint32 i) const {
 template <class T>
 inline
 void
-ArrayPool<T>::getPtr(Ptr<T> & ptr, bool CrashOnBoundaryError){
+ArrayPool<T>::getPtr(Ptr<T> & ptr, bool CrashOnBoundaryError) const
+{
   Uint32 i = ptr.i;
   if(likely(i < size)){
     ptr.p = &theArray[i];
@@ -665,7 +604,8 @@ ArrayPool<T>::getPtr(Ptr<T> & ptr, bool CrashOnBoundaryError){
 template <class T>
 inline
 void
-ArrayPool<T>::getPtr(ConstPtr<T> & ptr, bool CrashOnBoundaryError) const {
+ArrayPool<T>::getPtr(ConstPtr<T> & ptr, bool CrashOnBoundaryError) const
+{
   Uint32 i = ptr.i;
   if(likely(i < size)){
     ptr.p = &theArray[i];
@@ -688,7 +628,8 @@ ArrayPool<T>::getPtr(ConstPtr<T> & ptr, bool CrashOnBoundaryError) const {
 template <class T>
 inline
 void
-ArrayPool<T>::getPtr(Ptr<T> & ptr, Uint32 i, bool CrashOnBoundaryError){
+ArrayPool<T>::getPtr(Ptr<T> & ptr, Uint32 i, bool CrashOnBoundaryError) const
+{
   ptr.i = i;
   if(likely(i < size)){
     ptr.p = &theArray[i];
@@ -711,8 +652,10 @@ ArrayPool<T>::getPtr(Ptr<T> & ptr, Uint32 i, bool CrashOnBoundaryError){
 template <class T>
 inline
 void
-ArrayPool<T>::getPtr(ConstPtr<T> & ptr, Uint32 i, 
-		     bool CrashOnBoundaryError) const {
+ArrayPool<T>::getPtr(ConstPtr<T> & ptr,
+                     Uint32 i,
+                     bool CrashOnBoundaryError) const
+{
   ptr.i = i;
   if(likely(i < size)){
     ptr.p = &theArray[i];
@@ -735,7 +678,8 @@ ArrayPool<T>::getPtr(ConstPtr<T> & ptr, Uint32 i,
 template <class T>
 inline
 T * 
-ArrayPool<T>::getPtr(Uint32 i, bool CrashOnBoundaryError){
+ArrayPool<T>::getPtr(Uint32 i, bool CrashOnBoundaryError) const
+{
   if(likely(i < size)){
 #ifdef ARRAY_GUARD
     if (theAllocatedBitmask)
@@ -789,7 +733,8 @@ ArrayPool<T>::getConstPtr(Uint32 i, bool CrashOnBoundaryError) const {
 template <class T>
 inline
 void
-ArrayPool<T>::getPtrIgnoreAlloc(Ptr<T> & ptr){
+ArrayPool<T>::getPtrIgnoreAlloc(Ptr<T> & ptr) const
+{
   Uint32 i = ptr.i;
   if(likely (i < size)){
     ptr.p = &theArray[i];
@@ -1250,13 +1195,107 @@ error:
 #define DUMP(a,b)
 #endif
 
+
+template <class T>
+class CachedArrayPool : public ArrayPool<T>
+{
+public:
+//  typedef typename ArrayPool<T>::Cache Cache;
+  // typedef typename ArrayPool<T>::CallBack CallBack;
+  typedef void (CallBack)(CachedArrayPool<T>& pool);
+//  typedef typename ArrayPool<T>::LockFun LockFun;
+//  CachedArrayPool():ArrayPool<T>(NULL){}
+  explicit CachedArrayPool(CallBack* seizeErrorHandler=NULL)
+              : ArrayPool<T>(reinterpret_cast<typename ArrayPool<T>::CallBack*>(seizeErrorHandler)) { }
+
+  void setChunkSize(Uint32 sz);
+#ifdef ARRAY_CHUNK_GUARD
+  void checkChunks();
+#endif
+
+/***/
+  bool seize(Ptr<T> &p) { return ArrayPool<T>::seize(p); }
+  void release(Uint32 i) { return ArrayPool<T>::release(i); }
+  void releaseList(Uint32 n, Uint32 first, Uint32 last) { ArrayPool<T>::releaseList(n,first,last); }
+
+  /**
+   * Cache+LockFun is used to make thread-local caches for ndbmtd
+   *   I.e each thread has one cache-instance, and can seize/release on this
+   *       wo/ locks
+   */
+  struct Cache
+  {
+    Cache(Uint32 a0 = 512, Uint32 a1 = 256)
+    { m_first_free = RNIL; m_free_cnt = 0; m_alloc_cnt = a0; m_max_free_cnt = a1; }
+    void init_cache(Uint32 a0, Uint32 a1)
+    {
+      m_alloc_cnt = a0;
+      m_max_free_cnt = a1;
+    }
+    Uint32 m_first_free;
+    Uint32 m_free_cnt;
+    Uint32 m_alloc_cnt;
+    Uint32 m_max_free_cnt;
+  };
+
+  struct LockFun
+  {
+    void (* lock)(void);
+    void (* unlock)(void);
+  };
+
+  bool seize(LockFun, Cache&, Ptr<T> &);
+  void release(LockFun, Cache&, Uint32 i);
+  void release(LockFun, Cache&, Ptr<T> &);
+  void releaseList(LockFun, Cache&, Uint32 n, Uint32 first, Uint32 last);
+
+protected:
+  void releaseChunk(LockFun, Cache&, Uint32 n);
+
+  bool seizeChunk(Uint32 & n, Ptr<T> &);
+  void releaseChunk(Uint32 n, Uint32 first, Uint32 last);
+};
+
+template <class T>
+inline
+void
+CachedArrayPool<T>::setChunkSize(Uint32 sz)
+{
+  Uint32 const& size = this->size;
+  T * const& theArray = this->theArray;
+#ifdef ARRAY_GUARD
+  bool& chunk = this->chunk;
+#endif
+
+  Uint32 i;
+  for (i = 0; i + sz < size; i += sz)
+  {
+    theArray[i].chunkSize = sz;
+    theArray[i].lastChunk = i + sz - 1;
+    theArray[i].nextChunk = i + sz;
+  }
+
+  theArray[i].chunkSize = size - i;
+  theArray[i].lastChunk = size - 1;
+  theArray[i].nextChunk = RNIL;
+
+#ifdef ARRAY_GUARD
+  chunk = true;
+#endif
+
+#ifdef ARRAY_CHUNK_GUARD
+  checkChunks();
+#endif
+}
+
 #ifdef ARRAY_CHUNK_GUARD
 template <class T>
 inline
 void
-ArrayPool<T>::checkChunks()
+CachedArrayPool<T>::checkChunks()
 {
 #ifdef ARRAY_GUARD
+  bool const& chunk = this->chunk;
   assert(chunk == true);
 #endif
 
@@ -1287,9 +1326,17 @@ ArrayPool<T>::checkChunks()
 template <class T>
 inline
 bool
-ArrayPool<T>::seizeChunk(Uint32 & cnt, Ptr<T> & ptr)
+CachedArrayPool<T>::seizeChunk(Uint32 & cnt, Ptr<T> & ptr)
 {
+  Uint32 & firstFree = this->firstFree;
+  T* const& theArray = this->theArray;
 #ifdef ARRAY_GUARD
+  Uint32 & bitmaskSz = this->bitmaskSz;
+  Uint32 * const& theAllocatedBitmask = this->theAllocatedBitmask;
+#endif
+typename ArrayPool<T>::  CallBack const* const& seizeErrHand = this->seizeErrHand;
+#ifdef ARRAY_GUARD
+  bool const& chunk = this->chunk;
   assert(chunk == true);
 #endif
 
@@ -1322,7 +1369,7 @@ ArrayPool<T>::seizeChunk(Uint32 & cnt, Ptr<T> & ptr)
     } while (tmp > 0 && ff != RNIL);
     
     cnt = (save - tmp);
-    decNoFree(save - tmp);
+this->    decNoFree(save - tmp);
     firstFree = ff;
     theArray[prev].nextPool = RNIL;
     
@@ -1363,9 +1410,17 @@ ArrayPool<T>::seizeChunk(Uint32 & cnt, Ptr<T> & ptr)
 template <class T>
 inline
 void
-ArrayPool<T>::releaseChunk(Uint32 cnt, Uint32 first, Uint32 last)
+CachedArrayPool<T>::releaseChunk(Uint32 cnt, Uint32 first, Uint32 last)
 {
+  Uint32 & firstFree = this->firstFree;
+  T * const& theArray = this->theArray;
+  Uint32 & noOfFree = this->noOfFree;
 #ifdef ARRAY_GUARD
+  Uint32 & bitmaskSz = this->bitmaskSz;
+  Uint32 * const& theAllocatedBitmask = this->theAllocatedBitmask;
+#endif
+#ifdef ARRAY_GUARD
+  bool const& chunk = this->chunk;
   assert(chunk == true);
 #endif
 
@@ -1407,21 +1462,20 @@ ArrayPool<T>::releaseChunk(Uint32 cnt, Uint32 first, Uint32 last)
 #endif
 }
 
-
 template <class T>
 inline
 bool
-ArrayPool<T>::seize(LockFun l, Cache& c, Ptr<T> & p)
+CachedArrayPool<T>::seize(LockFun l, Cache& c, Ptr<T> & p)
 {
   DUMP("seize", "-> ");
 
   Uint32 ff = c.m_first_free;
   if (ff != RNIL)
   {
-    c.m_first_free = theArray[ff].nextPool;
+    c.m_first_free = this->theArray[ff].nextPool;
     c.m_free_cnt--;
     p.i = ff;
-    p.p = theArray + ff;
+    p.p = this->theArray + ff;
     DUMP("LOCAL ", "\n");
     return true;
   }
@@ -1433,14 +1487,14 @@ ArrayPool<T>::seize(LockFun l, Cache& c, Ptr<T> & p)
 
   if (ret)
   {
-    c.m_first_free = theArray[p.i].nextPool;
+    c.m_first_free = this->theArray[p.i].nextPool;
     c.m_free_cnt = tmp - 1;
     DUMP("LOCKED", "\n");
     return true;
   }
-  if (seizeErrHand != NULL)
+  if (this->seizeErrHand != NULL)
   {
-    (*seizeErrHand)(*this);
+    (*this->seizeErrHand)(*this);
   }
   return false;
 }
@@ -1448,17 +1502,17 @@ ArrayPool<T>::seize(LockFun l, Cache& c, Ptr<T> & p)
 template <class T>
 inline
 void
-ArrayPool<T>::release(LockFun l, Cache& c, Uint32 i)
+CachedArrayPool<T>::release(LockFun l, Cache& c, Uint32 i)
 {
   Ptr<T> tmp;
-  getPtr(tmp, i);
+this->  getPtr(tmp, i);
   release(l, c, tmp);
 }
 
 template <class T>
 inline
 void
-ArrayPool<T>::release(LockFun l, Cache& c, Ptr<T> & p)
+CachedArrayPool<T>::release(LockFun l, Cache& c, Ptr<T> & p)
 {
   p.p->nextPool = c.m_first_free;
   c.m_first_free = p.i;
@@ -1473,10 +1527,10 @@ ArrayPool<T>::release(LockFun l, Cache& c, Ptr<T> & p)
 template <class T>
 inline
 void
-ArrayPool<T>::releaseList(LockFun l, Cache& c,
+CachedArrayPool<T>::releaseList(LockFun l, Cache& c,
                           Uint32 n, Uint32 first, Uint32 last)
 {
-  theArray[last].nextPool = c.m_first_free;
+  this->theArray[last].nextPool = c.m_first_free;
   c.m_first_free = first;
   c.m_free_cnt += n;
 
@@ -1489,9 +1543,10 @@ ArrayPool<T>::releaseList(LockFun l, Cache& c,
 template <class T>
 inline
 void
-ArrayPool<T>::releaseChunk(LockFun l, Cache& c, Uint32 n)
+CachedArrayPool<T>::releaseChunk(LockFun l, Cache& c, Uint32 n)
 {
-  DUMP("releaseListImpl", "-> ");
+  T * const& theArray = this->theArray;
+  DUMP("releaseList", "-> ");
   Uint32 ff = c.m_first_free;
   Uint32 prev = ff;
   Uint32 curr = ff;
@@ -1521,18 +1576,19 @@ public:
    * Update p value for ptr according to i value 
    *   ignore if it's allocated or not
    */
-  void getPtrForce(Ptr<T> &);
+  void getPtrForce(Ptr<T> &) const;
   void getPtrForce(ConstPtr<T> &) const;
-  T * getPtrForce(Uint32 i);
+  T * getPtrForce(Uint32 i) const;
   const T * getConstPtrForce(Uint32 i) const;
-  void getPtrForce(Ptr<T> &, Uint32 i);
+  void getPtrForce(Ptr<T> &, Uint32 i) const;
   void getPtrForce(ConstPtr<T> &, Uint32 i) const;
 };
 
 template <class T>
 inline
 void 
-UnsafeArrayPool<T>::getPtrForce(Ptr<T> & ptr){
+UnsafeArrayPool<T>::getPtrForce(Ptr<T> & ptr) const
+{
   Uint32 i = ptr.i;
   if(likely(i < this->size)){
     ptr.p = &this->theArray[i];
@@ -1545,7 +1601,8 @@ UnsafeArrayPool<T>::getPtrForce(Ptr<T> & ptr){
 template <class T>
 inline
 void 
-UnsafeArrayPool<T>::getPtrForce(ConstPtr<T> & ptr) const{
+UnsafeArrayPool<T>::getPtrForce(ConstPtr<T> & ptr) const
+{
   Uint32 i = ptr.i;
   if(likely(i < this->size)){
     ptr.p = &this->theArray[i];
@@ -1558,7 +1615,8 @@ UnsafeArrayPool<T>::getPtrForce(ConstPtr<T> & ptr) const{
 template <class T>
 inline
 T * 
-UnsafeArrayPool<T>::getPtrForce(Uint32 i){
+UnsafeArrayPool<T>::getPtrForce(Uint32 i) const
+{
   if(likely(i < this->size)){
     return &this->theArray[i];
   } else {
@@ -1571,7 +1629,8 @@ UnsafeArrayPool<T>::getPtrForce(Uint32 i){
 template <class T>
 inline
 const T * 
-UnsafeArrayPool<T>::getConstPtrForce(Uint32 i) const {
+UnsafeArrayPool<T>::getConstPtrForce(Uint32 i) const
+{
   if(likely(i < this->size)){
     return &this->theArray[i];
   } else {
@@ -1584,7 +1643,8 @@ UnsafeArrayPool<T>::getConstPtrForce(Uint32 i) const {
 template <class T>
 inline
 void 
-UnsafeArrayPool<T>::getPtrForce(Ptr<T> & ptr, Uint32 i){
+UnsafeArrayPool<T>::getPtrForce(Ptr<T> & ptr, Uint32 i) const
+{
   ptr.i = i;
   if(likely(i < this->size)){
     ptr.p = &this->theArray[i];
@@ -1598,7 +1658,8 @@ UnsafeArrayPool<T>::getPtrForce(Ptr<T> & ptr, Uint32 i){
 template <class T>
 inline
 void 
-UnsafeArrayPool<T>::getPtrForce(ConstPtr<T> & ptr, Uint32 i) const{
+UnsafeArrayPool<T>::getPtrForce(ConstPtr<T> & ptr, Uint32 i) const
+{
   ptr.i = i;
   if(likely(i < this->size)){
     ptr.p = &this->theArray[i];
