@@ -1,17 +1,24 @@
-/* Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2008, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */
 
 
 #include "ConfigManager.hpp"
@@ -34,6 +41,13 @@ extern EventLogger * g_eventLogger;
 extern "C" const char* opt_ndb_connectstring;
 extern "C" int opt_ndb_nodeid;
 
+#if defined VM_TRACE || defined ERROR_INSERT
+extern int g_errorInsert;
+#define ERROR_INSERTED(x) (g_errorInsert == x)
+#else
+#define ERROR_INSERTED(x) false
+#endif
+
 ConfigManager::ConfigManager(const MgmtSrvr::MgmtOpts& opts,
                              const char* configdir) :
   MgmtThread("ConfigManager"),
@@ -235,6 +249,7 @@ ConfigManager::find_nodeid_from_config(void)
     return 0;
   }
 
+  delete conf;
   return found_nodeid;
 }
 
@@ -298,17 +313,17 @@ reset_dynamic_ports_in_config(const Config* config)
   for(;iter.valid();iter.next()) {
     Uint32 port;
     require(iter.get(CFG_CONNECTION_SERVER_PORT, &port) == 0);
-    
+
     if ((int)port < 0)
     {
       port = 0;
       ConfigValues::Iterator i2(config->m_configValues->m_config,
                                 iter.m_config);
       require(i2.set(CFG_CONNECTION_SERVER_PORT, port));
-    } 
+    }
   }
 }
- 
+
 
 bool
 ConfigManager::init(void)
@@ -333,8 +348,20 @@ ConfigManager::init(void)
   if (!init_nodeid())
     DBUG_RETURN(false);
 
-  if (m_opts.initial && !delete_saved_configs())
-    DBUG_RETURN(false);
+  if (m_opts.initial)
+  {
+    /**
+     * Verify valid -f before delete_saved_configs()
+     */
+    Config* conf = load_config();
+    if (conf == NULL)
+      DBUG_RETURN(false);
+
+    delete conf;
+
+    if (!delete_saved_configs())
+      DBUG_RETURN(false);
+  }
 
   if (failed_config_change_exists())
     DBUG_RETURN(false);
@@ -413,7 +440,7 @@ ConfigManager::init(void)
         DBUG_RETURN(false);
       }
 
-      /* 
+      /*
         The fetched config may contain dynamic ports for
         ndbd(s) which have to be reset to 0 before using
         the config
@@ -541,7 +568,7 @@ ConfigManager::prepareConfigChange(const Config* config)
     return false;
   }
 
-#ifdef __WIN__
+#ifdef _WIN32
   /*
 	File is opened with the commit flag "c" so
 	that the contents of the file buffer are written
@@ -1339,6 +1366,12 @@ ConfigManager::execCONFIG_CHECK_REQ(SignalSender& ss, SimpleSignal* sig)
 
   Uint32 generation = m_config->getGeneration();
 
+  if (ERROR_INSERTED(100) && nodeId != ss.getOwnNodeId())
+  {
+    g_eventLogger->debug("execCONFIG_CHECK_REQ() ERROR_INSERTED(100) => exit()");
+    exit(0);
+  }
+
   // checksum
   Uint32 checksum = config_check_checksum(m_config);
   Uint32 other_checksum = req->checksum;
@@ -1394,7 +1427,7 @@ ConfigManager::execCONFIG_CHECK_REQ(SignalSender& ss, SimpleSignal* sig)
   case CS_INITIAL:
     if (other_state != CS_INITIAL)
     {
-      g_eventLogger->warning("Refusing CONGIG_CHECK_REQ from %u, "
+      g_eventLogger->warning("Refusing CONFIG_CHECK_REQ from %u, "
                              "  it's not CS_INITIAL (I am). "
                              " Waiting for my check",
                              nodeId);
@@ -1715,7 +1748,9 @@ void
 ConfigManager::set_facade(TransporterFacade * f)
 {
   m_facade = f;
-  m_ss = new SignalSender(f, MGM_CONFIG_MAN);
+  m_ss = new SignalSender(f,
+                          MGM_CONFIG_MAN,  // blockNum
+                          true);           // deliverAll
   require(m_ss != 0);
 }
 
@@ -1781,7 +1816,7 @@ ConfigManager::run()
   assert(m_facade);
   SignalSender & ss = * m_ss;
 
-  if (!m_opts.config_cache)   
+  if (!m_opts.config_cache)
   {
     /* Stop receiving signals by closing ConfigManager's
        block in TransporterFacade */
@@ -1794,10 +1829,10 @@ ConfigManager::run()
     m_config_state = CS_CONFIRMED;
     ndbout_c("== ConfigManager disabled -- manager thread will exit ==");
     return;
-  }  
+  }
 
   ss.lock();
-  
+
   // Build bitmaks of all mgm nodes in config
   m_config->get_nodemask(m_all_mgm, NDB_MGM_NODE_TYPE_MGM);
 
@@ -1921,7 +1956,7 @@ ConfigManager::run()
         CAST_CONSTPTR(NFCompleteRep, sig->getDataPtr());
       NodeId nodeId= rep->failedNodeId;
 
-      if (m_all_mgm.get(nodeId)) // Not mgm node
+      if (!m_all_mgm.get(nodeId)) // Not mgm node
         break;
 
       ndbout_c("Node %d failed", nodeId);
@@ -2078,7 +2113,7 @@ ConfigManager::fetch_config(void)
 
   if (tmp == NULL) {
     g_eventLogger->error("%s", m_config_retriever.getErrorString());
-    DBUG_RETURN(false);
+    DBUG_RETURN(NULL);
   }
 
   DBUG_RETURN(new Config(tmp));
@@ -2313,7 +2348,7 @@ ConfigManager::get_packed_config(ndb_mgm_node_type nodetype,
         error.assign("get_packed_config, failed to set dynamic ports in config");
         return false;
       }
-      
+
       if (!config_copy.pack64(m_packed_config))
       {
         error.assign("get_packed_config, failed to pack config_copy");
@@ -2549,24 +2584,51 @@ check_dynamic_port_configured(const Config* config,
 
 bool
 ConfigManager::set_dynamic_port(int node1, int node2, int value,
-                                BaseString& msg){
+                                BaseString& msg)
+{
+  MgmtSrvr::DynPortSpec port = { node2, value };
 
+  return set_dynamic_ports(node1, &port, 1, msg);
+}
+
+
+bool
+ConfigManager::set_dynamic_ports(int node, MgmtSrvr::DynPortSpec ports[],
+                                 unsigned num_ports, BaseString &msg)
+{
   Guard g(m_config_mutex);
-  if (!check_dynamic_port_configured(m_config,
-                                     node1, node2, msg))
-    return false;
 
-  if (!m_dynamic_ports.set(node1, node2, value))
+  // Check that all ports to set are configured as dynamic
+  for(unsigned i = 0; i < num_ports; i++)
   {
-    msg.assfmt("Could not set dynamic port for %d -> %d", node1, node2);
-    return false;
+    const int node2 = ports[i].node;
+    if (!check_dynamic_port_configured(m_config,
+                                       node, node2, msg))
+    {
+      return false;
+    }
+  }
+
+  // Set the dynamic ports
+  bool result = true;
+  for(unsigned i = 0; i < num_ports; i++)
+  {
+    const int node2 = ports[i].node;
+    const int value = ports[i].port;
+    if (!m_dynamic_ports.set(node, node2, value))
+    {
+      // Failed to set one port, report problem but since it's very unlikley
+      // that this step fails, continue and attempt to set remaining ports.
+      msg.assfmt("Failed to set dynamic port(s)");
+      result =  false;
+    }
   }
 
   // Removed cache of packed config, need to be recreated
   // to include the new dynamic port
   m_packed_config.clear();
 
-  return true;
+  return result;
 }
 
 
@@ -2648,8 +2710,8 @@ ConfigManager::DynamicPorts::set_in_config(Config* config)
       continue; // Not configured as dynamic port
 
     Uint32 n1, n2;
-    require(iter.get(CFG_CONNECTION_NODE_1, &n1) == 0 &&
-            iter.get(CFG_CONNECTION_NODE_2, &n2) == 0);
+    require(iter.get(CFG_CONNECTION_NODE_1, &n1) == 0);
+    require(iter.get(CFG_CONNECTION_NODE_2, &n2) == 0);
 
     int dyn_port;
     if (!get(n1, n2, &dyn_port) || dyn_port == 0)
