@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2007, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -188,19 +195,21 @@ NdbInterpretedCode::add3(Uint32 x1, Uint32 x2, Uint32 x3)
 inline int 
 NdbInterpretedCode::addN(Uint32 *data, Uint32 length)
 {
-  if (unlikely(! have_space_for(length)))
-    return error(TooManyInstructions);
+  if (likely(length > 0))
+  {
+    if (unlikely(! have_space_for(length)))
+      return error(TooManyInstructions);
   
-  /* data* may be unaligned, so we do a byte copy
-   * using memcpy
-   */
-  memcpy(&m_buffer[m_instructions_length],
-         data,
-         length << 2);
+    /* data* may be unaligned, so we do a byte copy
+     * using memcpy
+     */
+    memcpy(&m_buffer[m_instructions_length],
+           data,
+           length << 2);
 
-  m_instructions_length+= length;
-  m_available_length-= length;
-  
+    m_instructions_length += length;
+    m_available_length -= length;
+  }
   return 0;
 }
 
@@ -514,12 +523,9 @@ NdbInterpretedCode::branch_col(Uint32 branch_type,
     DBUG_RETURN(error(BadAttributeId));
   }
 
-  Uint32 sigLen= len;
-  Uint32 sendLen= len;
   Uint32 lastWordMask= ~0;
-
   if (val == NULL)
-    sigLen = sendLen = 0;
+    len = 0;
   else {
     if (! col->getStringType())
     {
@@ -532,9 +538,9 @@ NdbInterpretedCode::branch_col(Uint32 branch_type,
         Uint32 bitLen= col->getLength();
         Uint32 lastWordBits= bitLen & 0x1F;
         if (lastWordBits)
-          lastWordMask= (1 << lastWordBits) -1;
+          lastWordMask = ((Uint32)1 << lastWordBits) -1;
       }
-      sigLen= sendLen= col->m_attrSize * col->m_arraySize;
+      len= col->m_attrSize * col->m_arraySize;
     }
     else
     {
@@ -545,7 +551,7 @@ NdbInterpretedCode::branch_col(Uint32 branch_type,
       if ((branch_type != Interpreter::LIKE) &&
           (branch_type != Interpreter::NOT_LIKE))
       {
-        if (! col->get_var_length_bug39645(val, sigLen, sendLen))
+        if (! col->get_var_length(val, len))
         {
           DBUG_RETURN(error(BadLength));
         }
@@ -556,30 +562,21 @@ NdbInterpretedCode::branch_col(Uint32 branch_type,
   if (col->m_storageType == NDB_STORAGETYPE_DISK)
     m_flags|= UsesDisk;
 
-  Uint32 tempData[ NDB_MAX_TUPLE_SIZE_IN_WORDS ];
-
-  if (sigLen != sendLen)
-  {
-    // bug39645
-    memcpy(tempData, val, sigLen);
-    val = tempData;
-  }
-
   if (add_branch(Interpreter::BranchCol(c, 0, 0), Label) != 0)
     DBUG_RETURN(-1);
 
-  if (add1(Interpreter::BranchCol_2(attrId, sendLen)) != 0)
+  if (add1(Interpreter::BranchCol_2(attrId, len)) != 0)
     DBUG_RETURN(-1);
 
   /* Get value byte length rounded up to nearest 32-bit word */
-  Uint32 len2 = Interpreter::mod4(sendLen);
-  if((len2 == sendLen) && (lastWordMask == (Uint32)~0))
+  Uint32 len2 = Interpreter::mod4(len);
+  if((len2 == len)  && (lastWordMask == (Uint32)~0))
   {
     /* Whole number of 32-bit words */
     DBUG_RETURN(addN((Uint32*)val, len2 >> 2));
   } 
 
-  /** else */
+  /* else */
   /* Partial last word */
   len2 -= 4;
   if (addN((Uint32*)val, len2 >> 2) != 0)
@@ -587,8 +584,7 @@ NdbInterpretedCode::branch_col(Uint32 branch_type,
   
   /* Zero insignificant bytes in last word */
   Uint32 tmp = 0;
-  for (Uint32 i = 0; i < sendLen-len2; i++)
-  {
+  for (Uint32 i = 0; i < len-len2; i++) {
     char* p = (char*)&tmp;
     p[i] = ((char*)val)[len2+i];
   }
@@ -718,7 +714,7 @@ NdbInterpretedCode::interpret_exit_nok(Uint32 ErrorCode)
 int
 NdbInterpretedCode::interpret_exit_nok()
 {
-  return add1((899 << 16) | Interpreter::EXIT_REFUSE);
+  return add1((626 << 16) | Interpreter::EXIT_REFUSE);
 }
 
 int
@@ -775,12 +771,12 @@ NdbInterpretedCode::add_val(Uint32 attrId, Uint64 aValue)
   {
     if (aValue < (1 << 16))
     {
-      if ((res= load_const_u16(7, aValue)) != 0)
+      if ((res= load_const_u16(7, (Uint32)aValue)) != 0)
         return res;  
     } 
     else
     {
-      if ((res= load_const_u32(7, aValue)) != 0)
+      if ((res= load_const_u32(7, (Uint32)aValue)) != 0)
         return res;
     }
   }
@@ -845,12 +841,12 @@ NdbInterpretedCode::sub_val(Uint32 attrId, Uint64 aValue)
   {
     if (aValue < (1 << 16))
     {
-      if ((res= load_const_u16(7, aValue)) != 0)
+      if ((res= load_const_u16(7, (Uint32)aValue)) != 0)
         return res;  
     } 
     else
     {
-      if ((res= load_const_u32(7, aValue)) != 0)
+      if ((res= load_const_u32(7, (Uint32)aValue)) != 0)
         return res;
     }
   }
@@ -975,6 +971,52 @@ NdbInterpretedCode::getWordsUsed() const
 
 }
 
+
+int
+NdbInterpretedCode::copy(const NdbInterpretedCode& src)
+{
+  m_table_impl = src.m_table_impl;
+  m_buffer_length = src.m_buffer_length;
+
+  /**
+   * Each NdbInterpretedCode manages life cycle of m_internal_buffer.
+   */
+  if (m_internal_buffer!=NULL)
+  {
+    delete[] m_internal_buffer;
+    m_internal_buffer = NULL;
+  }
+
+  if (src.m_internal_buffer==NULL)
+  {
+    // External buffer with externaly managed life cycle.
+    m_buffer = src.m_buffer;
+  }
+  else
+  {
+    m_buffer = m_internal_buffer = new Uint32[m_buffer_length];
+    if (unlikely(m_internal_buffer==NULL))
+    {
+      return 4000; // Alllocation failed.
+    }
+    memcpy(m_internal_buffer,
+           src.m_internal_buffer,
+           m_buffer_length*sizeof(Uint32));
+  }
+
+  m_number_of_labels = src.m_number_of_labels;
+  m_number_of_subs = src.m_number_of_subs;
+  m_number_of_calls = src.m_number_of_calls;
+  m_last_meta_pos = src.m_last_meta_pos;
+  m_instructions_length = src.m_instructions_length;
+  m_first_sub_instruction_pos = src.m_first_sub_instruction_pos;
+  m_available_length = src.m_available_length;
+  m_flags = src.m_flags;
+  m_error = src.m_error;
+  return 0;
+}
+
+
 /* CodeMetaInfo comparator for qsort 
  * Sort order is highest numbered sub to lowest,
  * then highest numbered label to lowest
@@ -1085,7 +1127,7 @@ NdbInterpretedCode::finalise()
 
       assert(info.type == Label);
       
-      Uint32 currOffset = ip - firstInstruction;
+      Uint32 currOffset = Uint32(ip - firstInstruction);
       Uint32 labelOffset= info.firstInstrPos;
       
       if (labelOffset >= m_instructions_length)
