@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -90,6 +90,7 @@ NdbOperation::NdbOperation(Ndb* aNdb, NdbOperation::Type aType) :
  *****************************************************************************/
 NdbOperation::~NdbOperation( )
 {
+  assert(theRequest == NULL);  // The same as theTCREQ
 }
 /******************************************************************************
  *void setErrorCode(int anErrorCode);
@@ -142,7 +143,6 @@ NdbOperation::setErrorCodeAbort(int anErrorCode) const
 int
 NdbOperation::init(const NdbTableImpl* tab, NdbTransaction* myConnection)
 {
-  NdbApiSignal* tSignal;
   theStatus		= Init;
   theError.code		= 0;
   theErrorLine		= 1;
@@ -167,11 +167,12 @@ NdbOperation::init(const NdbTableImpl* tab, NdbTransaction* myConnection)
   theCommitIndicator	= 0;
   theSimpleIndicator	= 0;
   theDirtyIndicator	= 0;
+  theReadCommittedBaseIndicator = 0;
   theInterpretIndicator	= 0;
   theDistrKeyIndicator_  = 0;
   theScanInfo        	= 0;
   theTotalNrOfKeyWordInSignal = 8;
-  theMagicNumber        = 0xABCDEF01;
+  theMagicNumber        = getMagicNumber();
   m_attribute_record= NULL;
   theBlobList = NULL;
   m_abortOption = -1;
@@ -181,8 +182,15 @@ NdbOperation::init(const NdbTableImpl* tab, NdbTransaction* myConnection)
   m_interpreted_code = NULL;
   m_extraSetValues = NULL;
   m_numExtraSetValues = 0;
+  m_customData = NULL;
 
-  tSignal = theNdb->getSignal();
+  if (theReceiver.init(NdbReceiver::NDB_OPERATION, this))
+  {
+    // theReceiver sets the error code of its owner
+    return -1;
+  }
+
+  NdbApiSignal* tSignal = theNdb->getSignal();
   if (tSignal == NULL)
   {
     setErrorCode(4000);
@@ -196,12 +204,6 @@ NdbOperation::init(const NdbTableImpl* tab, NdbTransaction* myConnection)
   tcKeyReq->scanInfo = 0;
   theKEYINFOptr = &tcKeyReq->keyInfo[0];
   theATTRINFOptr = &tcKeyReq->attrInfo[0];
-  if (theReceiver.init(NdbReceiver::NDB_OPERATION, this))
-  {
-    // theReceiver sets the error code of its owner
-    return -1;
-  }
-  m_customData = NULL;
 
   if (theNdb->theImpl->get_ndbapi_config_parameters().m_default_queue_option)
     m_flags |= OF_QUEUEABLE;
@@ -236,6 +238,13 @@ NdbOperation::release()
 
   theLockHandle = NULL;
   m_blob_lock_upgraded = false;
+
+#ifndef NDEBUG
+  // Poison members to detect late usage
+  m_accessTable = m_currentTable = (NdbTableImpl*) 0x1;
+  theNdbCon = (NdbTransaction*) 0x1;
+  m_key_record = m_attribute_record = (NdbRecord*) 0x1;
+#endif
 }
 
 void
