@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -52,7 +59,6 @@ Arguments:
 #include <ndb_global.h>
 #include "NdbApi.hpp"
 
-#include <NdbMain.h>
 #include <NdbOut.hpp>
 #include <NdbSleep.h>
 #include <NdbTick.h>
@@ -69,7 +75,7 @@ Arguments:
 #define MAXLONGKEYTOTALSIZE 1023 // words = 4092 bytes
 
 extern "C" { static void* flexBenchThread(void*); }
-static int readArguments(int argc, const char** argv);
+static int readArguments(int argc, char** argv);
 static int createTables(Ndb*);
 static int dropTables(Ndb*);
 static void sleepBeforeStartingTest(int seconds);
@@ -162,11 +168,11 @@ statReport(enum StartType st, int ops)
   static int nodeid;
   // open connection
   if (statState != statOpen) {
-    char *p = getenv("NDB_NODEID");		// ndbnet sets NDB_NODEID
+    char *p = getenv("NDB_NODEID");
     nodeid = p == 0 ? 0 : atoi(p);
     if ((statSock = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
       if (statState != statError) {
-	ndbout_c("stat: create socket failed: %s", strerror(errno));
+	ndbout_c("stat: create socket failed: %s", strerror(socket_errno));
 	statState = statError;
       }
       (void)NdbMutex_Unlock(&statMutex);
@@ -187,7 +193,7 @@ statReport(enum StartType st, int ops)
     }
     if (connect(statSock, (struct sockaddr *)&saddr, sizeof(saddr)) < 0) {
       if (statState != statError) {
-	ndbout_c("stat: connect failed: %s", strerror(errno));
+	ndbout_c("stat: connect failed: %s", strerror(socket_errno));
 	statState = statError;
       }
       (void)close(statSock);
@@ -225,9 +231,9 @@ statReport(enum StartType st, int ops)
   sprintf(buf, "%d %s %d\n", nodeid, text, ops);
   int len = strlen(buf);
   // assume SIGPIPE already ignored
-  if (write(statSock, buf, len) != len) {
+  if (send(statSock, buf, len, 0) != len) {
     if (statState != statError) {
-      ndbout_c("stat: write failed: %s", strerror(errno));
+      ndbout_c("stat: write failed: %s", strerror(socket_errno));
       statState = statError;
     }
     (void)close(statSock);
@@ -267,8 +273,11 @@ waitForThreads(ThreadData* pt)
     NdbSleep_MilliSleep(100);
     cont = 0;
     for (unsigned int i = 0; i < tNoOfThreads; i++){
-      if (pt[i].threadReady == 0) 
-    cont = 1;
+      if (pt[i].threadReady == 0){ 
+        // Found one thread not yet ready, continue waiting
+        cont = 1;
+        break;
+      }
     }
   }
 }
@@ -282,11 +291,11 @@ tellThreads(ThreadData* pt, StartType what)
 
 static Ndb_cluster_connection *g_cluster_connection= 0;
 
-NDB_COMMAND(flexBench, "flexBench", "flexBench", "flexbench", 65535)
+int main(int argc, char** argv)
 {
   ndb_init();
   ThreadData*           pThreadsData;
-  int                   tLoops = 0, i;
+  int                   tLoops = 0;
   int                   returnValue = NDBT_OK;
     
   if (readArguments(argc, argv) != 0){
@@ -367,11 +376,11 @@ NDB_COMMAND(flexBench, "flexBench", "flexBench", "flexbench", 65535)
      ****************************************************************/
     resetThreads(pThreadsData);
     
-    for (i = 0; i < (int)tNoOfThreads; i++){  
+    for (int i = 0; i < (int)tNoOfThreads; i++){  
       pThreadsData[i].threadNo = i;
       pThreadsData[i].threadLife = NdbThread_Create(flexBenchThread,
                                                     (void**)&pThreadsData[i],
-                                                    32768,
+                                                    64 * 1024,  // 64K stack
                                                     "flexBenchThread",
                                                     NDB_THREAD_PRIO_LOW);
     }
@@ -543,7 +552,7 @@ NDB_COMMAND(flexBench, "flexBench", "flexBench", "flexbench", 65535)
     waitForThreads(pThreadsData);
 
     void * tmp;
-    for(i = 0; i<(int)tNoOfThreads; i++){
+    for(int i = 0; i<(int)tNoOfThreads; i++){
       NdbThread_WaitFor(pThreadsData[i].threadLife, &tmp);
       NdbThread_Destroy(&pThreadsData[i].threadLife);
     }
@@ -552,7 +561,7 @@ NDB_COMMAND(flexBench, "flexBench", "flexBench", "flexbench", 65535)
   if (useLongKeys == true) {
     // Only free these areas if they have been allocated
     // Otherwise cores will happen
-    for (i = 0; i < (int)tNoOfLongPK; i++)
+    for (int i = 0; i < (int)tNoOfLongPK; i++)
       free(longKeyAttrName[i]);
     free(longKeyAttrName);
   } // if
@@ -652,6 +661,16 @@ static void* flexBenchThread(void* pArg)
   for (int tab= 0; tab<(int)tNoOfTables; tab++)
   {
     const NdbDictionary::Table *table= dict->getTable(tableName[tab]);
+    if (table == NULL)
+    {
+      // This is a fatal error, abort program
+      ndbout << "Failed to find table: " << tableName[tab];
+      ndbout << ", in thread: " << threadNo;
+      ndbout << endl;
+      tResult = 1; // Indicate fatal error
+      break;
+    }
+
     int numPKs= (useLongKeys ? tNoOfLongPK : 1);
 
     /* First create NdbRecord for just the primary key(s). */
@@ -905,7 +924,7 @@ static void* flexBenchThread(void* pArg)
                                                pAttrSet[countTables]);
 	  break;
 	default:
-	  assert(false);
+	  require(false);
 	}//switch
 
 	if (pOps[countTables] == NULL) {
@@ -942,7 +961,7 @@ static void* flexBenchThread(void* pArg)
 	  }//if
 	  break;
 	default:
-	  assert(false);
+	  require(false);
 	}//switch
       }//if
       tSpecialTrans = 0;
@@ -1074,7 +1093,7 @@ static void* flexBenchThread(void* pArg)
 }
 
 
-static int readArguments(int argc, const char** argv)
+static int readArguments(int argc, char** argv)
 {
 
   int i = 1;
@@ -1199,14 +1218,13 @@ static void sleepBeforeStartingTest(int seconds){
 
 static int
 createTables(Ndb* pMyNdb){
-  int i;
-  for (i = 0; i < (int)tNoOfAttributes; i++){
-    BaseString::snprintf(attrName[i], MAXSTRLEN, "COL%d", i);
+  for (Uint32 i = 0; i < tNoOfAttributes; i++){
+    BaseString::snprintf(attrName[i], MAXSTRLEN, "COL%u", i);
   }
 
   // Note! Uses only uppercase letters in table name's
   // so that we can look at the tables with SQL
-  for (i = 0; i < (int)tNoOfTables; i++){
+  for (Uint32 i = 0; i < tNoOfTables; i++){
     if (theStdTableNameFlag == 0){
       BaseString::snprintf(tableName[i], MAXSTRLEN, "TAB%d_%d", i, 
 	       (int)(NdbTick_CurrentMillisecond() / 1000));
@@ -1215,7 +1233,7 @@ createTables(Ndb* pMyNdb){
     }
   }
   
-  for(i = 0; i < (int)tNoOfTables; i++){
+  for(Uint32 i = 0; i < tNoOfTables; i++){
     ndbout << "Creating " << tableName[i] << "... ";
     
     NdbDictionary::Table tmpTable(tableName[i]);
