@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2011, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2011, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -809,7 +809,7 @@ NegativeTest::runKeyTest() const
       builder->destroy();
       return NDBT_FAILED;
     }
-    if (builder->prepare() != NULL)
+    if (builder->prepare(m_ndb) != NULL)
     {
       g_err << "prepare() on failed query gave non-NULL result.";
       builder->destroy();
@@ -825,7 +825,7 @@ NegativeTest::runKeyTest() const
       {builder->paramValue(), builder->paramValue(), NULL};
 
     require(builder->readTuple(m_nt1Tab, keyOperands) != NULL);
-    const NdbQueryDef* const queryDef = builder->prepare();
+    const NdbQueryDef* const queryDef = builder->prepare(m_ndb);
     require(queryDef != NULL);
     builder->destroy();
 
@@ -863,7 +863,7 @@ NegativeTest::runGraphTest() const
   // Try preparing empty NdbQueryBuilder
   {
     NdbQueryBuilder* const builder = NdbQueryBuilder::create();
-    if (builder->prepare() != NULL ||
+    if (builder->prepare(m_ndb) != NULL ||
         builder->getNdbError().code != QRY_HAS_ZERO_OPERATIONS)
     {
       g_err << "prepare() on empty query gave non-NULL result.";
@@ -1102,7 +1102,7 @@ NegativeTest::runSetBoundTest() const
       = builder->scanIndex(m_nt2OrdIdx, m_nt2Tab);
     require(parentOperation != NULL);
 
-    const NdbQueryDef* const queryDef = builder->prepare();
+    const NdbQueryDef* const queryDef = builder->prepare(m_ndb);
     require(queryDef != NULL);
     builder->destroy();
 
@@ -1161,7 +1161,7 @@ NegativeTest::runSetBoundTest() const
       = builder->scanIndex(m_nt1OrdIdx, m_nt1Tab);
     require(parentOperation != NULL);
 
-    const NdbQueryDef* const queryDef = builder->prepare();
+    const NdbQueryDef* const queryDef = builder->prepare(m_ndb);
     require(queryDef != NULL);
     builder->destroy();
 
@@ -1196,7 +1196,7 @@ NegativeTest::runSetBoundTest() const
       = builder->scanTable(m_nt1Tab);
     require(parentOperation != NULL);
 
-    const NdbQueryDef* const queryDef = builder->prepare();
+    const NdbQueryDef* const queryDef = builder->prepare(m_ndb);
     require(queryDef != NULL);
     builder->destroy();
 
@@ -1230,7 +1230,7 @@ NegativeTest::runSetBoundTest() const
       = builder->scanIndex(m_nt1OrdIdx, m_nt1Tab);
     require(parentOperation != NULL);
 
-    const NdbQueryDef* const queryDef = builder->prepare();
+    const NdbQueryDef* const queryDef = builder->prepare(m_ndb);
     require(queryDef != NULL);
     builder->destroy();
 
@@ -1282,7 +1282,7 @@ NegativeTest::runValueTest() const
       = builder->scanTable(m_nt1Tab);
     require(parentOperation != NULL);
 
-    const NdbQueryDef* const queryDef = builder->prepare();
+    const NdbQueryDef* const queryDef = builder->prepare(m_ndb);
     require(queryDef != NULL);
     builder->destroy();
 
@@ -1310,7 +1310,7 @@ NegativeTest::runValueTest() const
       = builder->scanTable(m_nt1Tab);
     require(parentOperation != NULL);
 
-    const NdbQueryDef* const queryDef = builder->prepare();
+    const NdbQueryDef* const queryDef = builder->prepare(m_ndb);
     require(queryDef != NULL);
     builder->destroy();
 
@@ -1342,7 +1342,7 @@ NegativeTest::runValueTest() const
       = builder->scanTable(m_nt1Tab);
     require(parentOperation != NULL);
 
-    const NdbQueryDef* const queryDef = builder->prepare();
+    const NdbQueryDef* const queryDef = builder->prepare(m_ndb);
     require(queryDef != NULL);
     builder->destroy();
 
@@ -1379,7 +1379,7 @@ NegativeTest::runValueTest() const
       = builder->scanIndex(m_nt1OrdIdx, m_nt1Tab);
     require(parentOperation != NULL);
 
-    const NdbQueryDef* const queryDef = builder->prepare();
+    const NdbQueryDef* const queryDef = builder->prepare(m_ndb);
     require(queryDef != NULL);
     builder->destroy();
 
@@ -1416,7 +1416,7 @@ NegativeTest::runFeatureDisabledTest() const
   
   int result = NDBT_OK;
 
-  if (ndb_join_pushdown(ndbGetOwnVersion()))
+  if (ndbd_join_pushdown(ndbGetOwnVersion()))
   {
     if (parentOperation == NULL)
     {
@@ -1505,12 +1505,36 @@ TESTCASE("MixedJoin", ""){
   STEPS(runJoin, 6);
   FINALIZER(runClearTable);
 }
+TESTCASE("MixedJoin17131", "Simulate CONTINUEB for DIGETNODESREQ"){
+  INITIALIZER(runLoadTable);
+  TC_PROPERTY("ErrorCode", 17131);
+  STEPS(runJoin, 6);
+  FINALIZER(runClearTable);
+}
 TESTCASE("MixedJoinDiskWait", "Simulate disk wait during pushed joins"){
   INITIALIZER(runLoadTable);
   TC_PROPERTY("ErrorCode", 4035);
   STEPS(runJoin, 4);
   FINALIZER(runClearTable);
 }
+TESTCASE("MultiFrag_OOM",
+         "'Out of LongMessageBuffer' during 'import' of MultiFrag list")
+{
+  INITIALIZER(runLoadTable);
+  TC_PROPERTY("ErrorCode", 8116);
+  TC_PROPERTY("AcceptError", 218);
+  STEP(runJoin);
+  FINALIZER(runClearTable);
+}
+TESTCASE("MultiFrag_OOM_rand",
+         "Random 'Out of LongMessageBuffer' during 'import' of MultiFrag list")
+{
+  INITIALIZER(runLoadTable);
+  TC_PROPERTY("ErrorCode", 8117);
+  TC_PROPERTY("AcceptError", 218);
+  STEP(runJoin);
+  FINALIZER(runClearTable);
+}
 TESTCASE("NF_Join", ""){
   TC_PROPERTY("UntilStopped", 1);
   TC_PROPERTY("WaitProgress", 20);
