@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2011, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,31 +22,22 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-#include <assert.h>
+#include <ndb_global.h>
+#include <ndb_opts.h>
+#include <NDBT.hpp>
+#include <NdbApi.hpp>
+
 #include <mysql.h>
 #include <mysqld_error.h>
 
 //#include <iostream>
 //#include <stdio.h>
 
-#include <ndb_global.h>
-#include <ndb_opts.h>
-#include <NDBT.hpp>
-#include <NdbApi.hpp>
 #include "../../src/ndbapi/NdbQueryBuilder.hpp"
 #include "../../src/ndbapi/NdbQueryOperation.hpp"
 #include <pthread.h>
 #include <NdbTick.h>
 
-
-
-#ifdef NDEBUG
-// Some asserts have side effects, and there is no other error handling anyway.
-#define ASSERT_ALWAYS(cond) if(unlikely(!(cond))){abort();}
-#else
-#define ASSERT_ALWAYS assert
-#endif
-
 #if 0
 /**
  * Helper debugging macros
@@ -67,12 +58,18 @@
 const char* databaseName = "PTDB";
 const char* tableName = "TT";
 
+enum JoinType
+{ lookupJoin,
+  scanLookupJoin,
+  scanScanJoin
+};
+
 class TestParameters{
 public:
   int m_iterations;
   /** Number of child lookup operations.*/
   int m_depth;
-  int m_scanLength; // m_scanLength==0 means root should be lookup.
+  int m_scanLength; // m_scanLength==0 implies 'lookupJoin'.
   /** Specifies how many times a query definition should be reused before.
    * It is recreated. Setting this to 0 means that the definition is never
    * recreated.*/ 
@@ -81,6 +78,8 @@ public:
   /** If true, run an equivalent SQL query.*/
   bool m_useSQL;
 
+  JoinType m_joinType;
+
   explicit TestParameters(){bzero(this, sizeof *this);}
 };
 
@@ -154,41 +153,42 @@ TestThread::TestThread(Ndb_cluster_connection& con,
                        int port): 
   m_params(NULL),
   m_ndb(&con, databaseName),
-  m_state(State_Active){
-  ASSERT_ALWAYS(m_ndb.init()==0);
-  ASSERT_ALWAYS(pthread_mutex_init(&m_mutex, NULL)==0);
-  ASSERT_ALWAYS(pthread_cond_init(&m_condition, NULL)==0);
-  ASSERT_ALWAYS(pthread_create(&m_posixThread, NULL, callback, this)
+  m_state(State_Active)
+{
+  require(m_ndb.init()==0);
+  require(pthread_mutex_init(&m_mutex, NULL)==0);
+  require(pthread_cond_init(&m_condition, NULL)==0);
+  require(pthread_create(&m_posixThread, NULL, callback, this)
                 ==0);
   NdbDictionary::Dictionary*  const dict = m_ndb.getDictionary();
   m_tab = dict->getTable(tableName);
 
   m_index = dict->getIndex("PRIMARY", tableName);
-  ASSERT_ALWAYS(m_index != NULL);
+  require(m_index != NULL);
 
   /* Create NdbRecord for row. */
   m_resultRec = m_tab->getDefaultRecord();
-  ASSERT_ALWAYS(m_resultRec!=NULL);
+  require(m_resultRec!=NULL);
 
   /* Create NdbRecord for primary key. */
   const NdbDictionary::Column *col1= m_tab->getColumn("a");
-  ASSERT_ALWAYS(col1 != NULL);
+  require(col1 != NULL);
   NdbDictionary::RecordSpecification spec = {
-    col1, 0, 0, 0
+    col1, 0, 0, 0, 0
   };
   
   m_keyRec = dict->createRecord(m_tab, &spec, 1, sizeof spec);
-  ASSERT_ALWAYS(m_keyRec != NULL);
+  require(m_keyRec != NULL);
 
   m_indexRec = m_index->getDefaultRecord();
-  ASSERT_ALWAYS(m_indexRec != NULL);
+  require(m_indexRec != NULL);
 
   // Make SQL connection.
-  ASSERT_ALWAYS(mysql_init(&m_mysql));
+  require(mysql_init(&m_mysql));
   if(!mysql_real_connect(&m_mysql, host, "root", "", "",
                          port, NULL, 0)){
     printMySQLError(m_mysql, "mysql_real_connect() failed:");
-    ASSERT_ALWAYS(false);
+    require(false);
   }
   char text[50];
   sprintf(text, "use %s", databaseName);
@@ -196,44 +196,43 @@ TestThread::TestThread(Ndb_cluster_connection& con,
 }
 
 TestThread::~TestThread(){
-  ASSERT_ALWAYS(pthread_mutex_lock(&m_mutex)==0);
+  require(pthread_mutex_lock(&m_mutex)==0);
   // Tell thread to stop.
   m_state = State_Stopping;
-  ASSERT_ALWAYS(pthread_cond_signal(&m_condition)==0);
+  require(pthread_cond_signal(&m_condition)==0);
   // Wait for thread to stop.
   while(m_state != State_Stopped){
-    ASSERT_ALWAYS(pthread_cond_wait(&m_condition, &m_mutex)==0);
+    require(pthread_cond_wait(&m_condition, &m_mutex)==0);
   }
-  ASSERT_ALWAYS(m_params == NULL);
-  ASSERT_ALWAYS(pthread_mutex_unlock(&m_mutex)==0);
-
-  ASSERT_ALWAYS(pthread_cond_destroy(&m_condition)==0);
-  ASSERT_ALWAYS(pthread_mutex_destroy(&m_mutex)==0);
+  require(m_params == NULL);
+  require(pthread_mutex_unlock(&m_mutex)==0);
+  require(pthread_cond_destroy(&m_condition)==0);
+  require(pthread_mutex_destroy(&m_mutex)==0);
 }
 
 void TestThread::start(const TestParameters& params){
-  ASSERT_ALWAYS(pthread_mutex_lock(&m_mutex)==0);
-  ASSERT_ALWAYS(m_params == NULL);
+  require(pthread_mutex_lock(&m_mutex)==0);
+  require(m_params == NULL);
   m_params = &params;
-  ASSERT_ALWAYS(pthread_cond_signal(&m_condition)==0);
-  ASSERT_ALWAYS(pthread_mutex_unlock(&m_mutex)==0);
+  require(pthread_cond_signal(&m_condition)==0);
+  require(pthread_mutex_unlock(&m_mutex)==0);
 }
 
 void TestThread::run(){
 
-  ASSERT_ALWAYS(pthread_mutex_lock(&m_mutex)==0);
+  require(pthread_mutex_lock(&m_mutex)==0);
   while(true){
     while(m_params==NULL && m_state==State_Active){
       // Wait for a new command from master thread.
-      ASSERT_ALWAYS(pthread_cond_wait(&m_condition, &m_mutex)==0);
+      require(pthread_cond_wait(&m_condition, &m_mutex)==0);
     }
     if(m_state != State_Active){
       // We have been told to stop.
-      ASSERT_ALWAYS(m_state == State_Stopping);
+      require(m_state == State_Stopping);
       m_state = State_Stopped;
       // Wake up master thread and release lock.
-      ASSERT_ALWAYS(pthread_cond_signal(&m_condition)==0);
-      ASSERT_ALWAYS(pthread_mutex_unlock(&m_mutex)==0);
+      require(pthread_cond_signal(&m_condition)==0);
+      require(pthread_mutex_unlock(&m_mutex)==0);
       // Exit thread.
       return;
     }
@@ -248,9 +247,9 @@ void TestThread::run(){
       }
     }
 
-    ASSERT_ALWAYS(m_params != NULL);
+    require(m_params != NULL);
     m_params = NULL;
-    ASSERT_ALWAYS(pthread_cond_signal(&m_condition)==0);
+    require(pthread_cond_signal(&m_condition)==0);
   }
 }
 
@@ -305,7 +304,7 @@ void TestThread::doLinkedAPITest(){
         };
         parentOpDef = builder->readTuple(m_tab, key);
       }
-      queryDef = builder->prepare();
+      queryDef = builder->prepare(&m_ndb);
     }
         
     if (!trans) {
@@ -322,7 +321,7 @@ void TestThread::doLinkedAPITest(){
     int res = trans->execute(NoCommit);
     //        if (res != 0)
     //          APIERROR(trans->getNdbError());
-    ASSERT_ALWAYS(res == 0);
+    require(res == 0);
     int cnt=0;
     while(true){
       const NdbQuery::NextResultOutcome outcome 
@@ -330,12 +329,12 @@ void TestThread::doLinkedAPITest(){
       if(outcome ==  NdbQuery::NextResult_scanComplete){
         break;
       }
-      ASSERT_ALWAYS(outcome== NdbQuery::NextResult_gotRow);
+      require(outcome== NdbQuery::NextResult_gotRow);
       cnt++;
       //        if (m_params->m_scanLength==0)
       //          break;
     }
-    ASSERT_ALWAYS(cnt== MAX(1,m_params->m_scanLength));
+    require(cnt== MAX(1,m_params->m_scanLength));
     //      query->close();
     if ((iterNo % 5) == 0) {
       m_ndb.closeTransaction(trans);
@@ -355,7 +354,7 @@ void TestThread::doNonLinkedAPITest(){
   for(int iterNo = 0; iterNo<m_params->m_iterations; iterNo++){
     //      NdbTransaction* const trans = m_ndb.startTransaction();
     if(m_params->m_scanLength>0){
-      const KeyRow highKey = { m_params->m_scanLength };
+      const KeyRow highKey = { (Uint32)m_params->m_scanLength };
       NdbIndexScanOperation* scanOp = NULL;
       if(m_params->m_scanLength==1){ // Pruned scan
         const NdbIndexScanOperation::IndexBound bound = {
@@ -393,9 +392,9 @@ void TestThread::doNonLinkedAPITest(){
                            NULL, // Result mask
                            &bound);
       }
-      ASSERT_ALWAYS(scanOp != NULL);
+      require(scanOp != NULL);
 
-      ASSERT_ALWAYS(trans->execute(NoCommit) == 0);
+      require(trans->execute(NoCommit) == 0);
           
       // Iterate over scan result
       int cnt = 0;
@@ -408,7 +407,7 @@ void TestThread::doNonLinkedAPITest(){
         if(retVal==1){
           break;
         }
-        ASSERT_ALWAYS(retVal== 0);
+        require(retVal== 0);
         //ndbout << "ScanRow: " << scanRow->a << " " << scanRow->b << endl;
         row = *scanRow;
             
@@ -421,15 +420,15 @@ void TestThread::doNonLinkedAPITest(){
                              m_resultRec,
                              reinterpret_cast<char*>(&row),
                              NdbOperation::LM_Dirty);
-          ASSERT_ALWAYS(lookupOp != NULL);
-          ASSERT_ALWAYS(trans->execute(NoCommit) == 0);
+          require(lookupOp != NULL);
+          require(trans->execute(NoCommit) == 0);
           //ndbout << "LookupRow: " << row.a << " " << row.b << endl;
         }
         cnt++;
         //          if (m_params->m_scanLength==0)
         //            break;
       }
-      ASSERT_ALWAYS(cnt== m_params->m_scanLength);
+      require(cnt== m_params->m_scanLength);
       scanOp->close(false,true);
     }else{ 
       // Root is lookup.
@@ -441,8 +440,8 @@ void TestThread::doNonLinkedAPITest(){
                            m_resultRec,
                            reinterpret_cast<char*>(&row),
                            NdbOperation::LM_Dirty);
-        ASSERT_ALWAYS(lookupOp != NULL);
-        ASSERT_ALWAYS(trans->execute(NoCommit) == 0);
+        require(lookupOp != NULL);
+        require(trans->execute(NoCommit) == 0);
       }
     }//if(m_params->m_isScan)
     //      m_ndb.closeTransaction(trans);
@@ -472,7 +471,7 @@ void TestThread::doSQLTest(){
 
   TextBuf text;
 
-  sprintf(text.tail(), "select * from ");
+  sprintf(text.tail(), "select straight_join * from ");
   for(int i = 0; i<m_params->m_depth+1; i++){
     sprintf(text.tail(), "%s t%d", tableName, i);
     if(i < m_params->m_depth){
@@ -491,8 +490,13 @@ void TestThread::doSQLTest(){
   }
 
   for(int i = 1; i<m_params->m_depth+1; i++){
-    // Compare primary key of Tn to attribute of Tn-1.
-    sprintf(text.tail(), "and t%d.b=t%d.a ", i-1, i);
+    if (m_params->m_joinType == scanLookupJoin){
+      // Compare primary key of Tn to attribute of Tn-1.
+      sprintf(text.tail(), "and t%d.b=t%d.a ", i-1, i);
+    }else{ //scanScanJoin
+      // Compare index column of Tn to attribute of Tn-1.
+      sprintf(text.tail(), "and t%d.b=t%d.a ", i, i-1);
+    }
   }
   if(printQuery){
     ndbout << text.m_buffer << endl;
@@ -504,11 +508,11 @@ void TestThread::doSQLTest(){
 }
 
 void TestThread::wait(){
-  ASSERT_ALWAYS(pthread_mutex_lock(&m_mutex)==0);
+  require(pthread_mutex_lock(&m_mutex)==0);
   while(m_params!=NULL){
-    ASSERT_ALWAYS(pthread_cond_wait(&m_condition, &m_mutex)==0);
+    require(pthread_cond_wait(&m_condition, &m_mutex)==0);
   }
-  ASSERT_ALWAYS(pthread_mutex_unlock(&m_mutex)==0);
+  require(pthread_mutex_unlock(&m_mutex)==0);
 }
 
 
@@ -516,11 +520,11 @@ void TestThread::wait(){
 
 static void makeDatabase(const char* host, int port, int rowCount){
   MYSQL mysql;
-  ASSERT_ALWAYS(mysql_init(&mysql));
+  require(mysql_init(&mysql));
   if(!mysql_real_connect(&mysql, host, "root", "", "",
                          port, NULL, 0)){
     printMySQLError(mysql, "mysql_real_connect() failed:");
-    ASSERT_ALWAYS(false);
+    require(false);
   }
   char text[200];
   sprintf(text, "create database if not exists %s", databaseName);
@@ -538,6 +542,12 @@ static void makeDatabase(const char* host, int port, int rowCount){
             i, (i+1)%rowCount);
     mySQLExec(mysql, text);
   }
+
+  sprintf(text, "create index ix on %s(b)", tableName);
+  mySQLExec(mysql, text);
+
+  sprintf(text, "analyze table %s", tableName);
+  mySQLExec(mysql, text);
 }
 
 static void printHeading(){
@@ -592,6 +602,7 @@ TestThread** threads = NULL;
 void warmUp(){
   ndbout << endl << "warmUp()" << endl;
   TestParameters param;
+  param.m_joinType = lookupJoin;
   param.m_useSQL = true;
   param.m_iterations = 10;
   param.m_useLinkedOperations = false;
@@ -614,6 +625,7 @@ void warmUp(){
 void testLookupDepth(bool useSQL){
   ndbout << endl << "testLookupDepth()" << endl;
   TestParameters param;
+  param.m_joinType = lookupJoin;
   param.m_useSQL = useSQL;
   param.m_iterations = 100;
   param.m_useLinkedOperations = false;
@@ -633,9 +645,32 @@ void testLookupDepth(bool useSQL){
   }
 }
 
-void testScanDepth(int scanLength, bool useSQL){
-  ndbout  << endl << "testScanDepth()" << endl;
+void testScanLookupDepth(int scanLength, bool useSQL=true){
+  ndbout  << endl << "testScanLookupDepth()" << endl;
   TestParameters param;
+  param.m_joinType = scanLookupJoin;
+  param.m_useSQL = useSQL;
+  param.m_iterations = 20;
+  param.m_useLinkedOperations = false;
+  param.m_scanLength = scanLength;
+  param.m_queryDefReuse = 0;
+  printHeading();
+  for(int i = 0; i<10; i++){
+    param.m_depth = i;
+    runTest(threads, threadCount, param);
+  }
+  printHeading();
+  param.m_useLinkedOperations = true;
+  for(int i = 0; i<10; i++){
+    param.m_depth = i;
+    runTest(threads, threadCount, param);
+  }
+}
+
+void testScanScanDepth(int scanLength, bool useSQL=true){
+  ndbout  << endl << "testScanScanDepth()" << endl;
+  TestParameters param;
+  param.m_joinType = scanScanJoin;
   param.m_useSQL = useSQL;
   param.m_iterations = 20;
   param.m_useLinkedOperations = false;
@@ -674,23 +709,24 @@ int main(int argc, char* argv[]){
   makeDatabase(host, port, 200);
   {
     Ndb_cluster_connection con(connectString);
-    ASSERT_ALWAYS(con.connect(12, 5, 1) == 0);
-    ASSERT_ALWAYS(con.wait_until_ready(30,30) == 0);
+    require(con.connect(12, 5, 1) == 0);
+    require(con.wait_until_ready(30,30) == 0);
 
-    const int threadCount = 1;
     threads = new TestThread*[threadCount];
     for(int i = 0; i<threadCount; i++){
       threads[i] = new TestThread(con, host, port);
     }
     sleep(1);
 
-    //testScanDepth(1);
-    //testScanDepth(2);
-    //testScanDepth(5);
+
+    //testScanLookupDepth(1);
+    //testScanLookupDepth(2);
+    //testScanLookupDepth(5);
     warmUp();
-    testScanDepth(50, true);
     testLookupDepth(true);
- 
+    testScanLookupDepth(50, true);
+    testScanScanDepth(50, true);
+
     for(int i = 0; i<threadCount; i++){
       delete threads[i];
     }
